package com.opencms.modules.listes;

import java.util.*;
import java.io.*;

// import javax.activation.*;

//import javax.mail.internet.*;
import javax.activation.*;
import javax.mail.*;
import javax.mail.internet.*;

import javax.naming.Context;
import javax.naming.InitialContext;

import com.opencms.core.A_OpenCms;
import com.opencms.core.CmsException;
import com.opencms.core.OpenCms;
import com.opencms.file.CmsObject;

/**
 *
 * Classe d'envoi de mail. Utilise javamail 1.2.
 * Ajoute un contrle au niveau de l'adresse du destinataire
 * (Erreurs smtp, adresses non valides, adresses mal formes,...)

 * @author Arnaud Colin
 */
 
public class Envoi {


	public Session session;

	public int id;
	public String[] to_BCC;
	
	// sera crase dans le cas d'un envoi hors liste de diffusion
	public boolean has_BCC = true;
	
	// la liste des adresses valides parmi celles figurant dans to
	public Vector to_valides;
	// la liste des adresses invalides parmi celles figurant dans to
	public Vector to_invalides;

	public static String to;
	public static  String from;
	public static  String host;

	public boolean debug = false;

	// le sujet et le corps du message...
	public String sujet;
	public String corps;

	boolean hasAttachedFile;
	public String fichier;

	// le rapport de resultat de l'envoi du mail (liste des erreurs, si des erreurs sont survenues)
	public boolean is_error_smtp;
	public boolean is_error_send;
	public String rapport;



/*******************************************************************
 * 
 * lancement de la procdure de rcupration des paramtres du module
 * 
 */

	// on charge les paramtres du module en lisant le registre
  	static {
		try {  	
			host = OpenCms.getRegistry().getModuleParameterString("com.opencms.modules.listes", "mail.smtp.host");
			from = OpenCms.getRegistry().getModuleParameterString("com.opencms.modules.listes", "MAIL_FROM");
			to = OpenCms.getRegistry().getModuleParameterString("com.opencms.modules.listes", "MAIL_TO");

			A_OpenCms.log("opencms_info", "Envoi.static :: rcupration des paramtres du module... host : "+host+ " - from " + from + " - to : "+to);
		}
		
		catch (Exception ex) {
		}
  	}


 	// Si les paramtres du modules sont modifis, le code suivant charge les nouvelles valeurs
  	static {
		try {
  			// set the pool
  			moduleParameterWasUpdated(null);
		} catch(CmsException  exc) {
  			// TODO: Error-handling
		}
  	}


/********************************************************************
*
* Les constructeurs pourl'objet Envoi
*
*********************************************************************/

	public Envoi () {

		// Initialisation de la session 'mail'  (rcupration de l'adresse du serveur smtp)
		init ();
	}

	public Envoi (String[] to_BCC, String from, String sujet, String corps, boolean debug) {
		// appel du constructeur par defaut
		this();

		this.id = 0;
		this.to_BCC = to_BCC;


		// par defaut, on suppose que toutes les adresses sont correctes
		// c'est le lancement d'une erreur qui videra le Vector
		this.to_valides = new Vector();
		for (int i=0; i<to_BCC.length; i++){
			to_valides.addElement(to_BCC[i]);
		}

		this.to_invalides = new Vector();

		this.sujet = sujet;
		this.corps = corps;

		/* fichiers attaches */
		this.hasAttachedFile = false;
		this.fichier = null;

		this.is_error_smtp = false;
		this.is_error_send = false;
		this.rapport = new String();

		this.debug = debug;
	 }


	/* un constructeur pour la gestion des fichiers attachs
	*/
	public Envoi (String[] to_BCC, String from, String sujet, String corps, String fichier, boolean b) {
		// appel du contructeur  5 paramtres
		this (to_BCC, from, sujet, corps, b);

		/* initialisation du chemin de fichier */
		this.hasAttachedFile = true;
		this.fichier = fichier;
	 }


	/* un constructeur pour initialiser la valeur de l'hte
	*/
	public Envoi (String[] to_BCC, String from, String sujet, String corps, boolean b, String host) {
		// appel du contructeur  5 paramtres
		this (to_BCC, from, sujet, corps, b);
		// initialisation de l'hte
		this.host = host;
	 }


 	/* un constructeur pour initialiser la valeur de l'identifiand du message
	*/
	public Envoi (int id, String[] to_BCC, String from, String sujet, String corps, boolean b, String host) {

		// appel du contructeur  5 paramtres
		this (to_BCC, from, sujet, corps, b);
		// initialisation de l'identifiant du message envoye
		this.id = id;

	 }



 	/* un constructeur pour l'envoi d'un message simple, de manire totalement annexe 
 	* a la liste de diffusion : 
 	* 	- expediteur
 	*	- destinataire
 	* 	- sujet
 	* 	- corps de texte
	*/
	public Envoi (String to, String from, String sujet, String corps) {
		
		// appel du constructeur par defaut (essentiellement pour initialiser la valeur 
		// du serveur smtp. Les autres variables seront ecrasees
		this ();
		
		this.to = to;
		// this.from = from;
		this.to_BCC = null;
		this.has_BCC = false;
		
		this.sujet = sujet;
		this.corps = corps;

		/* fichiers attaches */
		this.hasAttachedFile = false;
		this.fichier = null;

		this.is_error_smtp = false;
		this.is_error_send = false;
		this.rapport = new String();

		this.debug = debug;
	 }


 
 

  	/**
   	* This event-method is invoked if a user changes a parameter for a module in the module-
   	* administration.
   	* It reads the current name of the pool for this module and stores it to c_pool
   	* @param CmsObject - the cmsobject to get access to the cms.
   	*/
  	public static void moduleParameterWasUpdated(CmsObject cms) throws CmsException {

		host = OpenCms.getRegistry().getModuleParameterString("com.opencms.modules.listes", "mail.smtp.host");
		from = OpenCms.getRegistry().getModuleParameterString("com.opencms.modules.listes", "MAIL_FROM");
		to = OpenCms.getRegistry().getModuleParameterString("com.opencms.modules.listes", "MAIL_TO");

		if (to == null || to.equals(null) ) {
			to = "";
		}

		A_OpenCms.log("opencms_info", "Envoi.moduleParameterWasUpdated :: host : "+host+ " - from : "+from+ " - to : "+to);

  	}


/********************************************************************
*
* Les methodes pourl'objet Envoi
*
*********************************************************************/

public void init () {


	    try {
			// sauvegarde de l'adresse du serveur smtp dans la session d'envoi du mail
			Properties props = new Properties();
			props.put("mail.smtp.host", this.host);		
	
			String strDebug = String.valueOf(debug);
			if (debug) props.put("mail.debug", strDebug);
	
			Session session = Session.getDefaultInstance(props, null);
			session.setDebug(debug);
		}
		
		catch (Exception e) {
			try {
				com.opencms.modules.listes.Utils.log("Erreur sauvegarde des proprits : "+e.toString() );			
			}
			catch (Exception ex) {}
		}

	}



	 /*
	 * le processus d'envoi de messages, les contrles mis en place..
	 */
	 public void send() {

		// traces des parametres
		try {		
			com.opencms.modules.listes.Utils.log("Parametres de configuration (paramtres du module) : ");			
			com.opencms.modules.listes.Utils.log("\tsmtp : "+this.host);			
			com.opencms.modules.listes.Utils.log("\tExpediteur : "+this.from);			
			com.opencms.modules.listes.Utils.log("\tDestinataire : "+this.to);			
		}
		catch (Exception e) {
			A_OpenCms.log("opencms_info", "Envoi.send :: erreur : "+e.toString());
		}			


		try {
			// creation du message
			MimeMessage msg = new MimeMessage(session);

			/* mettre des contrles en place sue les adresses (voir consignes Alessio...)
			*  en particulier, vrifier que l'expditeur appartient bel et bien  la liste, qu'il n'appartient
			* pas  une blacklist
			*/
			msg.setFrom(new InternetAddress(this.from));

			// controle : la liste de diffusion n'est pas vide
			if ( this.has_BCC ) {
				/* envois  une liste de diffusion
				* vrification de chaque adresse (contrler si le destinataire n'appartient pas  une blacklist...
				*/
				InternetAddress[] address = new InternetAddress[this.to_BCC.length];
				for (int i=0; i<to_BCC.length; i++ ) {
					address[i] = new InternetAddress(this.to_BCC[i]);
				}
				msg.setRecipients(Message.RecipientType.BCC, address);
			} // end if 
			
			A_OpenCms.log("opencms_info", "Envoi.send :: to : "+this.to);

			msg.setRecipients(Message.RecipientType.TO, this.to);

			msg.setSubject(this.sujet);
			msg.setSentDate(new Date());

			/* association du contenu au mail :
			*	- si pice attach, dfinition des objets ncessaires  l'envoi de docnuments attachs
			*	- sinon, mise  jour du contenu du mail
			*/

			if (! hasAttachedFile ) {
				/* on peut galement recourir  la mthode setText (text, charset) si on connait le type */
				msg.setText(this.corps);
				msg.setDataHandler(new DataHandler(new ByteArrayDataSource(this.corps, "text/html")));
			}
			else {
				/* A IMPLEMENTER : ENVOI DE PLUSIEURS FICHIERS */

				/* attachement de la piece
				* cration et remplissage de la premiere partie de la pice
				*/
				MimeBodyPart mbp1 = new MimeBodyPart();

				// method setText : dediee a l'envoi de donnees 'text/plain'
				// mbp1.setText(this.corps);
				mbp1.setDataHandler(new DataHandler(new ByteArrayDataSource(this.corps, "text/html")));
				// mbp1.setDataHandler(new DataHandler(new ByteArrayDataSource(msgHTMLTEST, "text/html")));

				// creation de la seconde partie de la pice
				MimeBodyPart mbp2 = new MimeBodyPart();

				// attachement du fichier au message
				FileDataSource fds = new FileDataSource(this.fichier);
				mbp2.setDataHandler(new DataHandler(fds));
				mbp2.setFileName(fds.getName());

				// creation du multipart et attachement
				Multipart mp = new MimeMultipart();
				mp.addBodyPart(mbp1);
				mp.addBodyPart(mbp2);

				// ajout du contenu au message (pice jointe, corps de texte...)
				msg.setContent(mp);
			}

			try {
				com.opencms.modules.listes.Utils.log("RE-INITIALISATION : host avant send: "+this.host);			
			} catch (Exception ex){}
			
			Transport.send(msg);

			// pas de compte-rendu de l'operation dans le rapport
			// (une instance de l'objet Envoi ne correspond pas  l'envoi d'un mail
			//  toute la liste de diffusion mais  un paquet de messsages...)
		}

		catch (MessagingException mex) {
			// traage de l'erreur
			checkMessagingException (mex);
		}
	 }


	public void checkMessagingException (MessagingException mex) {
	
		try {
			com.opencms.modules.listes.Utils.log("Envoi.checkMessagingException : erreur : "+mex.toString());			
		}
		catch (Exception fex) {
		}

		// l'objet Vector est nettoy
		this.to_valides.clear();

		this.rapport = this.rapport + "\nErreur envoi mail : " + this.id;

		//ecriture dans le log
		try {
			com.opencms.modules.listes.Utils.log("Erreur envoi de mail...");			
		} catch (FileNotFoundException fnfe) {}			

		mex.printStackTrace();

	    Exception ex = mex;
	    do {

		if (ex instanceof SendFailedException) {

		    // existance d'adresses non valides
		    is_error_send = true;

		    SendFailedException sfex = (SendFailedException)ex;
		    Address[] invalid = sfex.getInvalidAddresses();

			if (invalid != null) {
				
				try {				
					com.opencms.modules.listes.Utils.log("Existence d'adresses invalides...");			
				}
				catch (FileNotFoundException fnfe) {}			

				this.rapport = this.rapport + "\n\tAdresses invalides : ";
				String add_inv = new String();

			    for (int i = 0; i < invalid.length; i++) {

					try {
						com.opencms.modules.listes.Utils.log("Adresse invalide : "+invalid[i]);			
					} catch (FileNotFoundException fnfe) {}			
	
					// traage des erreurs
					add_inv = add_inv + invalid[i] +"-";
	
					// ajout  la liste des adresses invalides
					this.to_invalides.addElement(invalid[i]);
			     } // end for
			     
			     // erreur consigne dans le rapport d'envoi
			     this.rapport = this.rapport + add_inv;
		    }

		    Address[] validUnsent = sfex.getValidUnsentAddresses();

			if (validUnsent != null) {
			    for (int i = 0; i < validUnsent.length; i++) {
			    	
			    	try {
					com.opencms.modules.listes.Utils.log("Adresse valide - courrier non envoy : "+validUnsent[i]);			
				} catch (FileNotFoundException fnfe) {}			
		    	
				// traage des erreurs
				String add_val_nonenv = new String();
				add_val_nonenv = "\n\tAdresses valides - courrier non envoye : " + validUnsent[i];

				// ajout  la liste des adresses valides
				this.to_valides.addElement(validUnsent[i]);

			    } //end for
			}

		    Address[] validSent = sfex.getValidSentAddresses();

			if (validSent != null) {
			    for (int i = 0; i < validSent.length; i++) {

					/* traage des erreurs */
					String add_val_env = new String();
	
					// erreur consignee dans le rapport d'envoi
					this.rapport = this.rapport + "\t" + add_val_env;
	
					// ajout  la liste des adresses valides
					this.to_valides.addElement(validSent[i]);

			    } // end for
		    }
		}
		else {
			is_error_smtp = true;
			this.rapport = this.rapport + ". Probleme smtp - verifier les paramtres de votre serveur de mails...";

			this.rapport = this.rapport + "Nature de l'erreur : "+ex;

				/* redaction des erreurs dans un fichier de log */
				try {
					com.opencms.modules.listes.Utils.log("Probleme smtp.");	
				}
				catch (FileNotFoundException fnfe) {}			
		}
		
		if (ex instanceof MessagingException)
		    ex = ((MessagingException)ex).getNextException();
		else
		    ex = null;
	    }
	   
	    while (ex != null);
		if (this.to_invalides.size() > 0 ){
			  reSend();
		}
	}


	/*
	* Si detection d'adresse(s) invalide(s), relance de la routine pour
	* les adresses valides :
	* La presence d'une seule adresse invalide dans le paquet d'adresse paralyse tout
	* le processus d'envoi du mail a toute la liste de diffusion
	*
	* NOTA : la validite d'une adresse porte sur la validite du destinataire pour un
	* nom de domaine existant. Dans le cas d'un nom de domaine inexistant, le mail est
	* envoy, l'expediteur recoit un message l'avertissant que le destinataire n'a pas
	* ete trouve.
	 */

	public void reSend() {

		/* lancement de la routine d'effacement des adresses invalides 
		* (lancee uniquement si la variable AUTO_DELETE_INVALID_ADRESSES est a true dans le
		* fichier de config server.xml
		*/

		 /* il faut prendre la moitie des elements du vecteur to_valide et en faire un tableau de String
		 * pour une raison qui m'echappe, la procedure de lancement de mail est relancee
		 * si une adresse invalide est detectee
		 */

		String[] to_relance = new String[this.to_valides.size()/2];
		for (int k=0; k<this.to_valides.size()/2; k++){
			Address add = (Address)this.to_valides.elementAt(k);
			to_relance[k] = add.toString();
		}

		Envoi envoiFiltre = new Envoi(to_relance, from, sujet, corps, debug);
	 	envoiFiltre.send();

		com.opencms.modules.listes.Utils.deleteInvalidEmails(to_invalides);

	}





/********************************************************************
*
* Fin des methodes pourl'objet Envoi
*
*********************************************************************/

	/*
	* NOTA : La gestion de la liste de diffusion ncessite un nouveau constructeur pour l'objet ENVOI
	* afin de permettre le passage en paramtre d'un tableau de hachage (Hashtable)
	*/


} // end class Envoi