package com.opencms.modules.listes;
import com.opencms.file.*;
import com.opencms.core.*;
import com.opencms.util.*;
import com.opencms.template.*;
import com.opencms.workplace.*;
import com.opencms.defaults.*;

import java.util.*;
import java.io.*;
import java.sql.*;
import java.text.*;

/**
 * Cette classe controle le template backoffice pour les utilisateurs inscrits aux listes
 * Auteur Alessio Zurleni
 * Version 1.0
 * Date 14/01/03
 */
public class listesAboBackoffice extends A_CmsBackoffice {

	/* une entre d'abonn comprend les informations suivantes :
		prenom	
		nom
		email		-> obligatoire
		text		
		sDate		-> date d'inscription  la liste
		channelName	-> le nom de la liste de diffusion		
	*/
	final static String 	LISTE_PRENOM	=	"prenom";
	final static String 	LISTE_NOM	=	"nom";
	final static String 	LISTE_EMAIL	=	"email";
	final static String 	LISTE_CHANNEL	=	"channelName";
	final static String 	LISTE_DATE	=	"date";
	final static String 	LISTE_TEXT	=	"text";		// un champ textuel utilis par l'admin




	/**
	 */
	public Class getContentDefinitionClass() {
		return listesAboContentDefinition.class;
	}

  	/**
   	* when clicking directly on one entry in the list ...
   	*/
   	/*
	public String getUrl(CmsObject cms, String tagcontent, A_CmsXmlContent doc,
    	Object userObject) throws Exception{
		return "system/modules/com.opencms.modules.homepage.news/" +
               "administration/news/EditBackoffice.html";
	}
	*/

	/**
	* Returns the URL of the dialog to add a new data entry.
	 */
	public String getCreateUrl(CmsObject cms, String tagcontent,
                                A_CmsXmlContent doc, Object userObject)
	throws Exception{
		return "system/modules/com.opencms.modules.listes/"
                + "administration/abonnes/EditBackoffice.html";
	}

	/**
	* 
	 */

	public String getBackofficeUrl(CmsObject cms, String tagcontent,
                                    A_CmsXmlContent doc, Object userObject)
	throws Exception {
		return "system/modules/com.opencms.modules.listes/"+
                "administration/abonnes/Backoffice.html";
	}

	/**
	* Returns the URL of the dialog to edit an existing data entry.
	 */

	public String getEditUrl(CmsObject cms, String tagcontent,
                             A_CmsXmlContent doc, Object userObject)
	throws Exception {
		return "system/modules/com.opencms.modules.listes/" +
               "administration/abonnes/EditBackoffice.html";
	}

	// Cette fonction est prpose  la modification d'un dataset (une entre correspondant  un abonn) existant

	public byte[] getContentEdit(CmsObject cms,
					CmsXmlWpTemplateFile template,
					String elementName,
					Hashtable parameters,
					String templateSelector)
	throws CmsException {
		boolean dateError = false;

    		// System.err.println("--- Edit -"+template.toString());

    		String error = "";
		GregorianCalendar actDate = new GregorianCalendar();
		// session will be created or fetched
		I_CmsSession session =
                (CmsSession)cms.getRequestContext().getSession(true);
		// get value of hidden input field action
		String action = (String) parameters.get("action");
        	if(action == null) action = "";
		//get value of id
		String id = (String) parameters.get("id");
		if (id == null) id = "";

		//System.err.println("getContentEdit:     id= "+id);
		// set existing data in the content definition object
		//& go to the done section of the template
    		if( action.equals("save") || action.equals("saveexit")) {
			//move to the done section in the template
			templateSelector = "done";

			//get data from the template
			int idIntValue2 = Integer.valueOf(id).intValue();
			String prenom	= (String) parameters.get(LISTE_PRENOM);
			String nom	= (String) parameters.get(LISTE_NOM);
			String email	= (String) parameters.get(LISTE_EMAIL);
			String sDate	= (String) parameters.get(LISTE_DATE);
			String text	= (String) parameters.get(LISTE_TEXT);	
			String channelName = (String)parameters.get(LISTE_CHANNEL);


			// create an Object to access the Id (l'objet channel fait rfrence au nom de la liste)
			listesChannelContentDefinition temp =
            			new listesChannelContentDefinition(channelName);
			int channelId = temp.getIntId();
			GregorianCalendar date = null;
			email.trim();
			try{
				date = listesAboContentDefinition.string2date(sDate);
			}catch(ParseException e) {
				System.err.println("[" + this.getClass().getName() + "] "
                        + e.getMessage());
				dateError = true;
			}


			listesAboContentDefinition editCD =
                    		new listesAboContentDefinition(cms, new  Integer(idIntValue2));

			// ensure something was filled in!
			if((email.equals("") || email == null || dateError == true)) {
				templateSelector = "default";
				template.setData(LISTE_PRENOM, prenom);
				template.setData(LISTE_NOM, nom);
				template.setData(LISTE_EMAIL, email);
				template.setData(LISTE_TEXT, text);
				template.setData("date",
                    		listesAboContentDefinition.date2string(actDate));
				// re-display data in selectbox
				session.putValue("checkSelectChannel", editCD.getChannel());
				if( email.equals("") || email == null || !listesAboContentDefinition.checkEmail(email)) {
            				error += template.getProcessedDataValue("missing", this, parameters) + "<br>";
					template.setData("error", error);
            				template.setData("emailHighlightStart", template.getData("redStart"));
            				template.setData("emailHighlightEnd", template.getData("redEnd"));
				}
				if(dateError == true) {
            				error += template.getProcessedDataValue("dateError", this, parameters) +"<br>";
            				template.setData("dateHighlightStart", template.getData("redStart"));
            				template.setData("dateHighlightEnd", template.getData("redEnd"));
					template.setData("error", error);
				}
				template.setData("setaction", "save");   // still not first time shown
				return startProcessing(cms, template, elementName, parameters, templateSelector);
			}
			

			//change the cd object content
			editCD.setPrenom(prenom);
			editCD.setNom(nom);
			editCD.setEmail(email);
			editCD.setText(text);
			editCD.setDate(date);
			editCD.setChannel(channelId);

			try {
				editCD.write(cms);
			} catch (Exception e) {
				System.err.println("listesAboEditBackoffice: couldnt update!");
				System.err.println("[" + this.getClass().getName() + "] " + e.getMessage());
				template.setData("error", e.toString());
				template.setData("email", email);
				template.setData("text", text);
				template.setData("date", listesAboContentDefinition.date2string(actDate));
				template.setData("setaction", "save");
				templateSelector = "default";
				return startProcessing(cms, template, elementName, parameters, templateSelector);
			}
		}
		//START: first time here and got valid id parameter:
		//		 Fill the template with data!
        	if(id != null && !id.equals("")) {
				//get data from cd object
				int idIntValue = Integer.valueOf(id).intValue();
				listesAboContentDefinition editCD = new listesAboContentDefinition(cms, new  Integer(idIntValue));

				//set the data from the CD in the template
				template.setData(LISTE_PRENOM, editCD.getPrenom());
				template.setData(LISTE_NOM, editCD.getNom());
				template.setData(LISTE_EMAIL, editCD.getEmail());
				template.setData(LISTE_TEXT, editCD.getText());
				template.setData(LISTE_DATE, editCD.getDate());
                		template.setData("setaction", "save");
				// re-display data in selectbox
				session.putValue("checkSelectChannel", editCD.getChannel());
			return startProcessing(cms, template, elementName, parameters, templateSelector);
		}
	

	//finally start the processing
	return startProcessing(cms, template, elementName, parameters, templateSelector);
	}


	// Cette fonction est prpose  la cration d'un dataset

	public byte[] getContentNew(CmsObject cms,
					CmsXmlWpTemplateFile template,
					String elementName,
					Hashtable parameters,
					String templateSelector)
	throws CmsException {
		// session will be created or fetched

		I_CmsSession session = (CmsSession) cms.getRequestContext().getSession(true);
		// get value of hidden input field action
		String action = (String) parameters.get("action");
    		if ( action == null) action = "";
		GregorianCalendar actDate = new GregorianCalendar();  // the actual Date
		boolean dateError = false;   // could the date-string be parsed?
		boolean channelError = false;
		int channelId = 0;
        	String error = "";

		//get value of id and the marker parameters.get("id");
		String id = (String) parameters.get("id");

		if (id == null) id = "";

		//no button pressed: go to the default section!
		if (action == null || action.equals("")) {
			templateSelector = "default";
			template.setData("setaction", "default");
            		// set actual Date
			template.setData("date", listesAboContentDefinition.date2string(actDate));
            		String firstname 	= "";
            		String lastname 	= "";
 
			//store marker in the session
			//confirmation button pressed, process data!
		} else {
			//create new data, therefore create new content definition instance
			//& go to the done section of the template
			templateSelector = "done";

			//read value of the inputfields
			String prenom 		= (String) parameters.get(LISTE_PRENOM);
			String nom 		= (String) parameters.get(LISTE_NOM);
			
			String email 		= (String) parameters.get(LISTE_EMAIL);
			String text 		= (String) parameters.get(LISTE_TEXT);
			String sDate		= (String) parameters.get(LISTE_DATE);
			String channelName 	= (String)parameters.get(LISTE_CHANNEL);

			if(channelName == null || channelName.equals(" ") || channelName.equals("") ) {
				channelError = true;
			}else {
				// create an ChannelObject to access the Id
				listesChannelContentDefinition temp = new listesChannelContentDefinition(channelName);
				channelId = temp.getIntId();
			}
			GregorianCalendar date = null;
			email.trim();
			try{
				date = listesAboContentDefinition.string2date(sDate);
			}catch(ParseException e) {
				System.err.println("getContentNew: ParseException");
				System.err.println("[" + this.getClass().getName() + "] " + e.getMessage());
				dateError = true;
			}
			// ensure something was filled in!
			if(	email.equals("") || email == null || !listesAboContentDefinition.checkEmail(email) || 
				dateError == true || channelError == true) {

					templateSelector = "default";
					template.setData(LISTE_PRENOM, prenom);
					template.setData(LISTE_NOM, nom);
					template.setData(LISTE_EMAIL, email);
					template.setData(LISTE_TEXT, text);
					template.setData("date", listesAboContentDefinition.date2string(actDate));
					// re-display data in selectbox
          				session.putValue("checkSelectChannel", ""+channelId);
					if( email.equals("") || email == null) {
            					error += template.getProcessedDataValue("missing", this, parameters) + "<br>";
						template.setData("error", error);
            					template.setData("emailHighlightStart", template.getData("redStart"));
            					template.setData("emailHighlightEnd", template.getData("redEnd"));
					}
					if(dateError == true) {
            					error += template.getProcessedDataValue("dateError", this, parameters) +"<br>";
            					template.setData("dateHighlightStart", template.getData("redStart"));
            					template.setData("dateHighlightEnd", template.getData("redEnd"));
						template.setData("error", error);
					}
					if(channelError == true) {
            					error += template.getProcessedDataValue("channelError", this, parameters) +"<br>";
            					template.setData("channelHighlightStart", template.getData("redStart"));
            					template.setData("channelHighlightEnd", template.getData("redEnd"));
						template.setData("error", error);
					}
					template.setData("setaction", "save");   // still not first time shown
					return startProcessing(cms, template, elementName, parameters, templateSelector);
			}

			//create new cd object with the fetched values of the inputfields
			listesAboContentDefinition newCD = new listesAboContentDefinition(email,prenom,nom,text,date,channelId);
			try {
				newCD.writeAutoIncrement(cms);
			} catch (Exception e) {
				System.err.println("listesAboInsertBackoffice: couldnt write!");
				System.err.println("[" + this.getClass().getName() + "] " + e.getMessage());
				template.setData("error", e.toString());
				template.setData(LISTE_PRENOM, prenom);
				template.setData(LISTE_NOM, nom);
				template.setData(LISTE_EMAIL, email);
                template.setData(LISTE_TEXT, text);
                template.setData(LISTE_DATE, listesAboContentDefinition.date2string(actDate));
				template.setData("setaction", "save");
				templateSelector = "default";
			}
		}

		// finally start the processing
		return startProcessing(cms, template, elementName, parameters, templateSelector);
	}


   
	


}

