package com.opencms.modules.listes;

import com.opencms.core.*;
import com.opencms.file.*;
import com.opencms.template.*;
import com.opencms.defaults.*;
import com.opencms.file.mySql.*;
import com.opencms.util.*;
import java.util.*;
import java.text.*;
import java.io.*;
import java.sql.*;
import java.net.*;               // URL

import javax.servlet.http.*;     // HttpServletRequest

/**
 * This is the Content Definition class for the listesAbo entries.
 * The class privides some constructors filter methods to access entries as well
 * as some methods to convert dates and strings.
 * Version 1.0
 * Date 14/01/03

 */
public class listesAboContentDefinition extends A_CmsContentDefinition implements I_CmsContent {

	// constants for day or month selection
	private static final Integer C_FLAG_DAY = new Integer(0);
	private static final Integer C_FLAG_MONTH = new Integer(1);

	private int intChannel = -1;

	// Une entre d'abonn. Il faudra respecter l'ordre des champs
	private int m_id 			= -1;		//	1
	private String m_email 			= "";		//	2
	private String m_prenom 		= "";		//	3
	private String m_nom 			= "";		//	4
	private String m_text 			= "";		//	5
	private GregorianCalendar m_date 	= null;		//	6
	private int m_channel 			= -1;		//	7
	private int m_lockstate 		= -1;		//	8

    	private static Locale m_locale = new Locale("Default", ""+Locale.getDefault());
    
	//private static Locale m_locale = new Locale("en", ""+Locale.UK);

	private boolean newElement = false;  // indicates if an element should be written or updated

	private static String c_pool = null;
	// private static NewsKeyAccess m_NewsKeyAccess = null;

	// Constants for table news_entry
	private static final String  C_SELECT_ALL = "select * from listes_abo";
	private static final String  C_INSERT_ENTRY = "insert into listes_abo values(?,?,?,?,?,?,?,?)";
	private static final String  C_DELETE_ENTRY = "delete from listes_abo where ID = ?";
	private static final String  C_COUNT = "select count(*) from listes_abo";
    	private static final String  C_SELECT_ID = "select * from listes_abo where ID = ?";
	private static final String  C_GET_ALL = "select * from listes_abo";
	private static final String  C_UPDATE_ENTRY = "update listes_abo set  email = ?,prenom = ?, nom = ?, text = ?, date = ?, channel = ?, lockstate = ? where id = ?";
	private static final String  C_ORDER = "select * from listes_abo order by ? desc";
	private static final String  C_SELECT_CHANNEL_ID = "select count(*) from listes_abo where channel = ?";
	private static final String  C_SELECT_MONTH = "select * from listes_abo where month(date)= ? and channel = ? order by date desc";
	private static final String  C_SELECT_DAY = "select * from listes_abo where (date between ? and ?) and (channel = ?)";
	private static final String  C_SELECT_FIRST_N_OF_CHANNEL = "select * from listes_abo where channel = ? order by date desc";

	// @param : arnaud colin : tri par ordre alphabtique...
	private static final String  C_SELECT_BY_CHANNEL_AND_EMAIL_LIKE = "select * from listes_abo where channel = ? and email LIKE '%arn%' "; //order by email
	// upper()...
	private static final String  C_SELECT_BY_CHANNEL_ = "select * from listes_abo where channel = ? "; //order by email


	// arnaud colin : requete de test dans la table liste_abo :
	// l'email passe en parametre existe t'il ?
	private static final String  C_SELECT_EMAIL = "select * from listes_abo where email = ?";
	private static final String  C_INSERT_EMAIL = "insert into listes_abo (email, prenom, nom, date) values(?,?,?,?)";
	private static final String  C_INSERT_EMAIL_NODATE = "insert into listes_abo (email, prenom, nom, channel, lockstate) values(?,?,?,?, '-1')";
	private static final String  C_DELETE_EMAIL = "delete from listes_abo where email = ?";
	private static final String  C_INSERT_ENTRY_AUTOINCREMENT =  "insert into listes_abo(email, prenom, nom, text, date, channel, lockstate) values(?,?,?,?,?,?,?)";


    public static final String	 C_TABLE_LISTEABO = "LISTES_ABO";
    public static final int      C_NOT_LOCKED = -1;


  	static {
    		try {
      			moduleParameterWasUpdated(null);
    		} catch(CmsException  exc) {
      			// TODO: Error-handling
    		}
  	}

  	/**
   	* This event-method is invoked if a user changes a parameter for a module in the module-
   	* administration.
   	* It reads the current name of the pool for this module and stores it to c_pool
   	* @param CmsObject - the cmsobject to get access to the cms.
   	*/
  	public static void moduleParameterWasUpdated(CmsObject cms) throws CmsException {
    		c_pool = OpenCms.getRegistry().getModuleParameterString("com.opencms.modules.listes", "poolname");
    		listesAboContentDefinition.updatePoolName(c_pool);
  	}


	/**
	 * Constructor
	 */
	public listesAboContentDefinition() {
		super();
	}

	/**
	 * Constructor to read an Element from the database
	 * is needed by A_Backoffice like this!
	 */
	public listesAboContentDefinition(CmsObject cms, Integer ID) throws CmsException {
		java.util.Date date = null;
		m_date = new GregorianCalendar();

   		Connection con = null;
		PreparedStatement statement = null;
		ResultSet res = null;
		try {
      			con = DriverManager.getConnection(c_pool);
		 	statement = con.prepareStatement(C_SELECT_ID);
			statement.setInt(1, ID.intValue());  // choose id of the Element  "where id == "
			res = statement.executeQuery();
			if(res.next()){
				// read the items from the resultset
				m_id = res.getInt(1);
				m_email = res.getString(2);
				m_prenom = res.getString(3);
				m_nom = res.getString(4);
				m_text = res.getString(5);
                		java.sql.Timestamp timestamp = res.getTimestamp(6);
				java.util.Date tempDate = (java.util.Date)timestamp;
				m_date.setTime( tempDate ); // SQLDate to Date to GregorianCalendar
				//System.err.println("NewsContentDefinition" + m_date.get(Calendar.DATE)+"." + m_date.get(Calendar.MONTH)+"." + m_date.get(Calendar.YEAR) );
				m_lockstate = res.getInt(8);
				m_channel = res.getInt(7);
			}
		}catch(SQLException e) {
			throw new CmsException("[" + this.getClass().getName() + "] " + e.getMessage(),
				CmsException.C_SQL_ERROR, e);
		}finally{
			try{
				res.close();
			}catch (Exception e) {
				// ignore the exception
			}
            		try{
				statement.close();
			}catch (Exception e) {
				// ignore the exception
			}
            		try{
				con.close();
			}catch (Exception e) {
				// ignore the exception
			}
		}
	}

	/**
	 * Constructor to create an Element
	 *
	 * @param name
	 * @param descr
	 */
	public listesAboContentDefinition(
			String email,
			String prenom,
			String nom,
			String text,
			GregorianCalendar date,
			int channel) {
		
		m_email		= email;
		m_prenom	= prenom;
		m_nom		= nom;
		m_text		= text;
		m_date		= date;
		m_channel	= channel;
		m_lockstate	= C_NOT_LOCKED; // default
		newElement 	= true;        	// this is a new Element
		try{
			m_id = com.opencms.dbpool.CmsIdGenerator.nextId(c_pool, C_TABLE_LISTEABO);
		} catch(CmsException e){
			
			System.err.println("[" + this.getClass().getName() + "] " + e.getMessage());
		}
	}

	/**
	 * Constructor to create an Element with a certain ID
	 * needed to create a list of Elementes
	 *
	 * @param id    The unique id
	 * @param name  The name
	 * @param descr The descr
	 */
	private listesAboContentDefinition(	int id,
						String email,
						String prenom,
						String nom,
						String text,
						java.sql.Timestamp date,
						int channel,
						int lockstate) {
		m_id		= id;
		m_email		= email;
		m_prenom	= prenom;
		m_nom		= nom;
		m_text		= text;
		m_date = new GregorianCalendar();
        	java.util.Date tmpDate = new java.util.Date(date.getTime());
	
        	m_date.setTime(tmpDate);
		m_channel = channel;
		m_lockstate = lockstate;
	}


	/**
	 * write an Element to the database
	 * @param cms An instance of CmsObject
	 */
	public void write(CmsObject cms) throws Exception{
        	Connection con = null;
		PreparedStatement statement = null;
		ResultSet res = null;
		java.util.Date uDate = m_date.getTime();  	// GregorianCalendar to Date
		
        	java.sql.Timestamp sqlDate = new java.sql.Timestamp(uDate.getTime());
        
		if(newElement == true) {
			// Element not jet in Database -> insert Statement
			try {
				// write the data to database
                		con = DriverManager.getConnection(c_pool);
		 		statement = con.prepareStatement(C_INSERT_ENTRY);
				statement.setInt(1, m_id);
				statement.setString(2, m_email);
				statement.setString(3, m_prenom);
				statement.setString(4, m_nom);
				statement.setString(5, m_text);
				statement.setTimestamp(6, sqlDate);
				statement.setInt(7, m_channel);
                		statement.setInt(8, m_lockstate);

				statement.execute();
				newElement = false; // element is no longer new!
			}catch(Exception e) {
				throw new CmsException("[" + this.getClass().getName() + "] " + e.getMessage(),
				CmsException.C_SQL_ERROR, e);
			}finally{
                		try{
				    statement.close();
			    	}catch (Exception e) {
                    			// ignore the exception
			    	}
                		try{
                    			con.close();
				}catch (Exception e) {
				    // ignore the exception
				}
			}

		}else {
			// Element is already in Database -> update statement
			update(cms, sqlDate);
		}
	}



	/**
	 * utilisation de l'autoincrement mysql pour l'insertion en base de donnees
	 * @author : arnaud colin
	 *  
	 */
	public void writeAutoIncrement(CmsObject cms) throws Exception{
        	Connection con = null;
		PreparedStatement statement = null;
		ResultSet res = null;
		java.util.Date uDate = m_date.getTime();  	// GregorianCalendar to Date
        	//clearcache(cms); // clear the cache to make changes visible
		
        	java.sql.Timestamp sqlDate = new java.sql.Timestamp(uDate.getTime());
        
		if(newElement == true) {
			// Element not jet in Database -> insert Statement
			try {
				// write the data to database
                		con = DriverManager.getConnection(c_pool);
		 		statement = con.prepareStatement(C_INSERT_ENTRY_AUTOINCREMENT);
				// statement.setInt(1, m_id);
				statement.setString(1, m_email);
				statement.setString(2, m_prenom);
				statement.setString(3, m_nom);
				statement.setString(4, m_text);
				statement.setTimestamp(5, sqlDate);
				statement.setInt(6, m_channel);
           		statement.setInt(7, m_lockstate);
			
				statement.execute();
				newElement = false; // element is no longer new!
			}catch(Exception e) {
				throw new CmsException("[" + this.getClass().getName() + "] " + e.getMessage(),
				CmsException.C_SQL_ERROR, e);
			}finally{
                		try{
				    statement.close();
			    	}catch (Exception e) {
                    			// ignore the exception
			    	}
                		try{
                    			con.close();
				}catch (Exception e) {
				    // ignore the exception
				}
			}

		}else {
			// Element is already in Database -> update statement
			update(cms, sqlDate);
		}
	}



	/**
	 * method to updata an existing entry.
	 * This method is only called by the write()-method.
	 * @param CmsObject
	 * @param the new date (java.sql.Date)
	 */
	private void update(CmsObject cms, java.sql.Timestamp sqlDate) throws CmsException {
        Connection con = null;
		PreparedStatement statement = null;
		ResultSet res = null;
		
		try {
			// write the data to database
            con = DriverManager.getConnection(c_pool);
		 	statement = con.prepareStatement(C_UPDATE_ENTRY);
			
			statement.setString(1, m_email);
			statement.setString(2, m_prenom);
			statement.setString(3, m_nom);
			statement.setString(4, m_text);
			statement.setTimestamp(5, sqlDate);
			statement.setInt(6, m_channel);
           	statement.setInt(7, m_lockstate);
			statement.setInt(8, m_id);

			statement.execute();

		}catch(Exception e) {
			throw new CmsException("[" + this.getClass().getName() + "] " + e.getMessage(),
				CmsException.C_SQL_ERROR, e);
		}finally{
            		try{
				statement.close();
			}catch (Exception e) {
				// ignore the exception
			}
            		try{
				con.close();
			}catch (Exception e) {
				// ignore the exception
			}
		}
	}

	/**
	 * delete an Element from the database
	 * TOTO : effectuer les contrles d'intgrit rfrentielle
	 * @param cms An instance of CmsObject
	 */
	public void delete(CmsObject cms) throws CmsException {
        	Connection con = null;
		PreparedStatement statement = null;
		ResultSet res = null;
        	//clearcache(cms); // clear the cache to make changes visible
		try {
			// delet data from database
            		con = DriverManager.getConnection(c_pool);
		 	statement = con.prepareStatement(C_DELETE_ENTRY);
			statement.setInt(1, m_id);
			statement.execute();
		}catch(Exception e) {
			throw new CmsException("[" + this.getClass().getName() + "] " + e.getMessage(),
			CmsException.C_SQL_ERROR, e);
		}finally{
			 try{
				statement.close();
			}catch (Exception e) {
				// ignore the exception
			}
            		try{
				con.close();
			}catch (Exception e) {
				// ignore the exception
			}
		}
	}

  	/**
	 * create a pool if necessary
	 *
	 * @return c_pool
	 */
	private static String getPoolName(CmsObject cms) {

		if(c_pool == null) {
      			try{
        			c_pool = cms.getRegistry().getModuleParameterString("com.opencms.modules.listes", "poolname");
      			} catch(CmsException exc) {
        			// TODO: Add error-handling here!
      			}
		}
		return c_pool;
	}


	/**
	*	The three methods getFieldNames(), getFieldMethods() and getFilterMethods() are used
		by the Backoffice classes to generate the lists of entries.
	*
	**/



	/**
	 * declare the entries that should be displayed in the Backoffice list
	 * @return a Vector
	 */
	public static Vector getFieldMethods(CmsObject cms) {
		Vector methods = new Vector();
		try {
			methods.addElement(listesAboContentDefinition.class.getMethod("getEmail", new Class[0]));
			methods.addElement(listesAboContentDefinition.class.getMethod("getPrenom", new Class[0]));
			methods.addElement(listesAboContentDefinition.class.getMethod("getNom", new Class[0]));
			methods.addElement(listesAboContentDefinition.class.getMethod("getText", new Class[0]));
			methods.addElement(listesAboContentDefinition.class.getMethod("getDate", new Class[0]));
			methods.addElement(listesAboContentDefinition.class.getMethod("getChannel", new Class[0]));
		}catch(NoSuchMethodException e) {
			System.err.println("Exception in listesAboContentDefinition.getFieldMethods(CmsObject)!"+ e.getMessage());
		}
		return methods;
	}

	/**
	 * This method returns the labels of the columns for the list of Content Definition entries generated by the Backoffice classes.
	 * @return a Vector
	 */
	public static Vector getFieldNames(CmsObject cms) {
		Vector names = new Vector();
		names.addElement("Email");
		names.addElement("Prenom");
		names.addElement("Nom");
		names.addElement("Text");
		names.addElement("Date");
		names.addElement("Channel");
		return names;
	}

	/**
	 * declare the methods to filter the list entries
	 * @return A Vector
	 */
	public static Vector getFilterMethods(CmsObject cms) {
		Vector filterMethods = new Vector();
        	String showText = "";
		Integer actChannelId = null;  // has to be Integer because of Reflection!
        	Vector channels = null;
        	GregorianCalendar actDate = new GregorianCalendar();
        	int actMonth = actDate.get(Calendar.MONTH) + 1;
    		try {
	  		channels = listesChannelContentDefinition.getChannelList(); // get all actual channels
    		}
    		catch(CmsException e) {
      			return null;
    		}
	  	// add the always shown first filterMethod...
		try {
			filterMethods.addElement(new CmsFilterMethod("Tous tris par...", listesAboContentDefinition.class.getMethod("getSortedList", new Class[] {String.class}), new Object[] {}));
		}catch(NoSuchMethodException e) {
			System.err.println("Exception: listesAboContentDefinition.getFilterMethods(CmsObject)!"+ e.getMessage());
		}
		// try to add some filters dynamically
		for(int i=0; i< channels.size(); i++) {
			showText = ((listesChannelContentDefinition)channels.elementAt(i)).getName();
			actChannelId = new Integer(((listesChannelContentDefinition)channels.elementAt(i)).getIntId());
			try {
				filterMethods.addElement(new CmsFilterMethod(showText +": date...", listesAboContentDefinition.class.getMethod("getList", new Class[] {Integer.class, String.class}), new Object[] {actChannelId}, ""));				
				filterMethods.addElement(new CmsFilterMethod(showText +": mois d'inscription..." ,  listesAboContentDefinition.class.getMethod("getDynamicList", new Class[] {Integer.class, Integer.class, String.class}), new Object[] {actChannelId, C_FLAG_MONTH}, ""+actMonth));
				filterMethods.addElement(new CmsFilterMethod(showText +": contenu mail..." ,  listesAboContentDefinition.class.getMethod("getEmailLikeList", new Class[] {Integer.class, String.class}), new Object[] {actChannelId}, ""));
			}
			catch(NoSuchMethodException e) {
				System.err.println("Exception: listesAboContentDefinition.getFilterMethods(CmsObject)!"+ e.getMessage());
			}
		}

		return filterMethods;
	}

	/**
	 * one filter method
	 * this method creates a list of all entries
	 *
	 * @return a Vector with all Elementes
	 */
	public static Vector getList(Integer channel, String stringN) {
		Vector list = new Vector();
        	Connection con = null;
		PreparedStatement statement = null;
		ResultSet res = null;
		int i =  0;
		int n = -1;
		
		try{
			n = Integer.parseInt(stringN);
		}catch(NumberFormatException e) {
			//System.err.println("Exception in listesAboContentDefinition.getNewsList()!"+e.getMessage());
			n = -1;
		}
		try {
            		con = DriverManager.getConnection(c_pool);
		 	statement = con.prepareStatement(C_SELECT_FIRST_N_OF_CHANNEL);
			statement.setInt(1, channel.intValue());
			res = statement.executeQuery();
			while(res.next() && (i != n)){
				// read the item from the resultset
				// insert Element into list
				list.addElement(new listesAboContentDefinition(res.getInt(1),
				res.getString(2),  // Email
				res.getString(3),  // Prenom
				res.getString(4),  // Nom
				res.getString(5),  // Text
				res.getTimestamp(6),    // Date
				res.getInt(7),  	// Channel
				res.getInt(8)    	// Lockstate
				));
				i++;
			}
		}catch(Exception e) {
			System.err.println("Exception in listesAboContentDefinition.getList()!");
		}finally{
			try{
				res.close();
			}catch (Exception e) {
				// ignore the exception
			}
            		try{
				statement.close();
			}catch (Exception e) {
				// ignore the exception
			}
            		try{
				con.close();
			}catch (Exception e) {
				// ignore the exception
			}
		}
		return list;
	}


	/**
	 * one filter method
	 * Permet (apparamment) de trier selon un champ donn (le nom du champs doit correspondre)
	 * Ne tient pas compte du fichier des langues
	 *
	 * @return a Vector with all Elementes
	 */

	public static Vector getSortedList(String str) {
		Vector list = new Vector();
        	Connection con = null;
		PreparedStatement statement = null;
		ResultSet res = null;
		if(str == null || str == " ") { str = ""; }
		str.trim();
		str.toLowerCase();
		// System.err.println("[listesAboContentDefinition.getSortedList()] Start - str: " +str);
		try {
            		con = DriverManager.getConnection(c_pool);
		 	statement = con.prepareStatement(C_ORDER);
			if( str.equals("") ) {
				statement.setString(1, "date");
			}
			else if( str.equals("id") ) {
				statement.setString(1, "id");
			}
			else if(str.equals("email") || str.equals("e-mail") ) {
				statement.setString(1, "email");
			}
			else if(str.equals("prenom") || str.equals("prnom")) {
				statement.setString(1, "prenom");
			}
			else if(str.equals("nom") ) {
				statement.setString(1, "nom");
			}
			else if(str.equals("date") ) {
				statement.setString(1, "date");
			}
			else if(str.equals("channel") ) {
				statement.setString(1, "channel");
			}
			else {
				statement.setString(1, "date");	// all other inputs! par dfaut

				// on effectue le tri sur la chaine de caractres passs en paramtres
				// System.out.println("paramtre : " + str);
				
			}
			res = statement.executeQuery();
			while(res.next()){
				// read the item from the resultset
				// insert Element into list
				list.addElement(new listesAboContentDefinition(res.getInt(1),
				res.getString(2),  // Email
				res.getString(3),  // Prenom
				res.getString(4),  // Nom
				res.getString(5),  // Text
				res.getTimestamp(6),    // Date
				res.getInt(7),    	// Channel
				res.getInt(8)  	// Lockstate

				));
			}
			
		}catch(Exception e) {
			System.err.println("[listesAboContentDefinition.getSortedList() Exception1:]" + e.getMessage());
		}finally{
			try{
				res.close();
			}catch (Exception e) {
				// ignore the exception
			}
            		try{
				statement.close();
			}catch (Exception e) {
				// ignore the exception
			}
            		try{
				con.close();
			}catch (Exception e) {
				// ignore the exception
			}
		}
		return list;
	}

 
	/**
	 * one filter method
	 * @return a Vector with all selected Elementes
	 */
	public static Vector getDynamicList(Integer channelId, Integer flag, String str) throws CmsException {

		Vector list 			= new Vector();
		java.sql.Timestamp day 		= null;
        	GregorianCalendar dayEnd 	= new GregorianCalendar();
        	Connection con 			= null;
		PreparedStatement statement 	= null;
		ResultSet res 			= null;
		int month 			= 1;
		//System.err.println("[listesAboContentDefinition.getDynamicList()] Start - channelId:" +""+channelId.intValue() + "flag: " + ""+flag.intValue() +"str: " + str);
		if(flag.equals(C_FLAG_MONTH)) {
			try{
				month = Integer.parseInt(str);
			}catch(NumberFormatException e) {
				System.err.println("[listesAboContentDefinition.getDynamicList()] Exception while trying to parse the month: "+e.getMessage());
			}
			if(month <1 || month >12) month = 1;
		}
		else
		{
			// "day"
			if( (str == null) || (str.equals("")) || (str.equals(" "))) {
				try{
					//day = shortString2sqlTimestamp("01.01.2000"); // this default value seems to be needed by A_Backoffice
					day = date2Timestamp(dayEnd);
                    			dayEnd.set(Calendar.HOUR, 23);
                    			dayEnd.set(Calendar.MINUTE, 59);
                    			dayEnd.set(Calendar.SECOND, 59);
                		}catch(Exception e) {
					System.err.println("[NewsContentDefinition.getDynamicList()] Exception while trying to parse the date: "+e.getMessage());
				}
			}else {
				try{;
					day = shortString2sqlTimestamp(str);  // now a real date is parsed
                    			dayEnd = shortString2date(str);
                    			dayEnd.set(Calendar.HOUR, 23);
                    			dayEnd.set(Calendar.MINUTE, 59);
                    			dayEnd.set(Calendar.SECOND, 59);
				}catch(ParseException e) {
					System.err.println("[NewsContentDefinition.getDynamicList()] Exception while trying to parse the date: "+e.getMessage());
				}
			}
		}
			

		try{
            			con = DriverManager.getConnection(c_pool);
				if(flag.equals(C_FLAG_MONTH)){
					statement = con.prepareStatement(C_SELECT_MONTH);
					statement.setInt(1, month);
					statement.setInt(2, channelId.intValue());
				}else if(flag.equals(C_FLAG_DAY)){
					// "day"
					statement = con.prepareStatement(C_SELECT_DAY);
					statement.setTimestamp(1, day);
                			statement.setTimestamp(2, date2Timestamp(dayEnd) );
					statement.setInt(3, channelId.intValue());
				}
				else {
					
					// System.out.println("channel : "+channelId.intValue() );

				}
				res = statement.executeQuery();
				while(res.next()){
					// read the item from the resultset
					// insert Element into list
					list.addElement(new listesAboContentDefinition(res.getInt(1),
						res.getString(2),  // Email
						res.getString(3),  // Prenom
						res.getString(4),  // Nom
						res.getString(5),  // Text
						res.getTimestamp(6),    // Date
						res.getInt(7),  	// Channel
						res.getInt(8)    	// Lockstate
					));

				}
		}catch(Exception e) {
      				throw new CmsException(e.getMessage());
				//System.err.println("[NewsContentDefinition.getDynamicList() Exception while trying to execute the statement:]" + e.getMessage());
		}finally{
            		try{
				res.close();
			}catch (Exception e) {
				// ignore the exception
			}
			try{
				statement.close();
			}catch (Exception e) {
				// ignore the exception
			}
            		try{
				con.close();
			}catch (Exception e) {
				// ignore the exception
			}
		}
			
		return list;

	}






	/**
	 * Emails tris par liste de diffusion et email like
	 *
	 * @param out renvoi un vecteur
	 */
	public static Vector getEmailLikeList(Integer channel, String stringN) {


	// upper()...
		
		Vector list = new Vector();
        	Connection con = null;
		PreparedStatement statementS = null;
		ResultSet res = null;


		try {
            		con = DriverManager.getConnection(c_pool);
            		
            		String req = new String();

            		
            		if (stringN == null || stringN == "" || stringN.equals(null) || stringN.equals("nul") || stringN.equals("") ) {
				req = "select * from listes_abo where channel = "+channel.intValue();
				req = req + " order by email desc";
			 	// System.out.println( "req "+req);
			}
			else {
				req = "select * from listes_abo where channel = "+channel.intValue();
				req = req + " and email like '%"+stringN+"%'";
				req = req + " order by email desc";			 	
			}
		 	
		 	statementS = con.prepareStatement(req);
			res = statementS.executeQuery();
			while( res.next() ){
				// lecture du resultset - instanciation nouvel objet listesAboContentDefinition - insertion base de donnes
				list.addElement(new listesAboContentDefinition(res.getInt(1),
				res.getString(2),  // Email
				res.getString(3),  // Prenom
				res.getString(4),  // Nom
				res.getString(5),  // Text
				res.getTimestamp(6),    // Date
				res.getInt(7),  	// Channel
				res.getInt(8)    	// Lockstate
				));
			}
		}
		catch(Exception e) {
			System.err.println("Exception in listesAboContentDefinition.getEmailLikeList()!");
			System.err.println("Erreur : "+e.toString() );
			
		}
		finally{
			try{
				res.close();
			}catch (Exception e) {
				// ignore the exception
			}
            		try{
				statementS.close();
			}catch (Exception e) {
				// ignore the exception
			}
            		try{
				con.close();
			}catch (Exception e) {
				// ignore the exception
			}
		}
		
		return list;
	}





	/**
	 * This method indicates if a channel with a certain ID is used by a news entry.
	 * If this is so, the channel should not be deleted.
	 * @param the channel Id
	 * @return true if the channel is used
	 */
	public static boolean channelIsUsed(int channelId) {
		//System.err.println("[NewsContentDefinition.channelIsUsed()] channelId" + ""+channelId);
    		Connection con = null;
		PreparedStatement statement = null;
		ResultSet res = null;
		int ret = -1;
		try {
            con = DriverManager.getConnection(c_pool);
		 	statement = con.prepareStatement(C_SELECT_CHANNEL_ID);
			statement.setInt(1, channelId);
			res = statement.executeQuery();
			if(res.next()){
				ret = res.getInt(1);
				//System.err.println("channelIsUsed: " + res.getInt(1) );
			}
			//System.err.println("channelIsUsed: " + res.getInt(1) );
		}catch(SQLException e) {
			System.err.println("Exception in NewsContentDefinition.channelIsUsed()!");
		}finally{
			try{
				res.close();
			}catch (Exception e) {
				// ignore the exception
			}
			try{
				statement.close();
			}catch (Exception e) {
				// ignore the exception
			}
            try{
				con.close();
			}catch (Exception e) {
				// ignore the exception
			}
			if(ret > 0) return true;
			else return false;
		}
	}

	/**
	 * sets the lockstate of the CD.
	 * @param lockstate The lockstate of this CD object.
	 * (possibles states: locked by you,locked by other, unlocked)
	 */
	public void setLockstate(int lockstate) {
		m_lockstate = lockstate;
	}

	/**
	 * gets the lockstate of the CD
	* (possibles states: locked by user,locked by other, unlocked)
	*/
	public int getLockstate() {
		return m_lockstate;
	}

	/**
	* if the content should be lockable
	* the method returns true
	* @returns a boolean
	*/
	public static boolean isLockable() {
		return true;
	}

	/**
	 * get the unique Id (for abstract Backoffice class)
	 * @return m_id The unique id (as a String)
	 */
	public String getUniqueId(CmsObject cms) {
		return ""+m_id;
	}

	/**
	 * get the Unique Id as int
	 * @return m_id (as int)
	 */
	public int getIntId() {
		return m_id;
	}

	/**
	 * get the Email
	 * @return the email as String
	 */
	public String getEmail() {
		return m_email;
	}

	/**
	 * set the Email
	 * @param the Email as String
	 */
	public void setEmail(String email) {
		m_email = email;
	}

	/**
	 * get the prenom
	 * @return the prenom as String
	 */
	public String getPrenom() {
		return m_prenom;
	}


/*****************************************************************************
*
*	Methodes statiques
*
******************************************************************************/

	/**
	 * enregistrement d'un abonne dans la liste de diffusion : 
	 */
	public static void write(String email, String prenom, String nom, GregorianCalendar gregDate) throws Exception {
        	

		java.util.Date uDate = gregDate.getTime(); 
        	java.sql.Timestamp sqlDate = new java.sql.Timestamp(uDate.getTime());


    		Connection con = null;
		PreparedStatement statement1 = null;
		ResultSet res = null;

		
		try {
   			con = DriverManager.getConnection(c_pool);

		 	statement1 = con.prepareStatement(C_INSERT_EMAIL);

			statement1.setString(1, email);
			statement1.setString(2, prenom);
			statement1.setString(3, nom);
			statement1.setTimestamp(4, sqlDate);

			res = statement1.executeQuery();
		}
		
		catch(SQLException e) {
			// lancement si erreur au niveau requete sql
		}
		finally{
			try{
				res.close();
			}catch (Exception e) {
				// ignore the exception
			}
            		try{
				statement1.close();
			}catch (Exception e) {
				// ignore the exception
			}
            		try{
				con.close();
			}catch (Exception e) {
				// ignore the exception
			}
		}
	}
			
			



	/**
	 * enregistrement d'un abonne dans la liste de diffusion : 
	 * MEME FONCTION SANS LA DATE
	 * 
	 * NOTA : l'acces a ces fonctions se fait depuis l'exterieur de la plateforme, 
	 *	et les variables permettant la connexion a la base de donnees ne semblent 
	 * 	pas accessibles (classes non chargees par le serveur d'app)
	 * 		=> codage en dur de l'url et du driver (pour le moment...)
	 * 
	 */
	public static void write (String email, String prenom, String nom, int channel) throws Exception {

    		Connection con = null;
		PreparedStatement statement1 = null;
		ResultSet res = null;

	
		try {
			con = DriverManager.getConnection(c_pool);

		 	statement1 = con.prepareStatement(C_INSERT_EMAIL_NODATE);

			statement1.setString(1, email);
			statement1.setString(2, prenom);
			statement1.setString(3, nom);
			statement1.setInt(4, channel);
			res = statement1.executeQuery();
		}
		catch(SQLException e) {
			// lancement si erreur au niveau requete sql
			System.out.println("Erreur sql : "+e.toString() );
		}
		finally{
			try{
				res.close();
			}catch (Exception e) {
				// ignore the exception
			}
            		try{
				statement1.close();
			}catch (Exception e) {
				// ignore the exception
			}
            		try{
				con.close();
			}catch (Exception e) {
				// ignore the exception
			}
		}
	}


	/**
	 * effacement d'un abonne de la liste de diffusion
	 * fonction appele depuis le formulaire d'inscription/dsinscription...
	 */

	public static void delete (String email) throws Exception {

    		Connection con = null;
		PreparedStatement statement1 = null;
		ResultSet res = null;

		try {
			con = DriverManager.getConnection(c_pool);

		 	statement1 = con.prepareStatement(C_DELETE_EMAIL);

			statement1.setString(1, email);
			res = statement1.executeQuery();
			
		}
		catch(SQLException e) {
			// lancement si erreur au niveau requete sql
			System.out.println("Erreur sql : "+e.toString() );
		}
		finally{
			try{
				res.close();
			}catch (Exception e) {
				// ignore the exception
			}
            		try{
				statement1.close();
			}catch (Exception e) {
				// ignore the exception
			}
            		try{
				con.close();
			}catch (Exception e) {
				// ignore the exception
			}
		}
	}



	/**
	 * Requete sur la table listes_abo :
	 * l'email passe en parametre existe t'il deja dans la table ?
	 */
	public static boolean emailAlreadyExists (String email) {

    		
    		boolean alreadyExists = false;

    		Connection con = null;
		PreparedStatement statement1 = null;
		ResultSet res = null;


		try {
            con = DriverManager.getConnection(c_pool);

		 	statement1 = con.prepareStatement(C_SELECT_EMAIL);
			statement1.setString(1, email); 
			res = statement1.executeQuery();
			
			if(res.next()){
				// read the items from the resultset
				int id = res.getInt(1);
				alreadyExists = true;
			}
		}catch(SQLException e) {
			System.out.println("erreur : "+e.toString());
			// lancement si erreur au niveau requete sql
		}finally{
			try{
				res.close();
			}catch (Exception e) {
				// ignore the exception
			}
            		try{
				statement1.close();
			}catch (Exception e) {
				// ignore the exception
			}
            		try{
				con.close();
			}catch (Exception e) {
				// ignore the exception
			}
		}
		return alreadyExists;
		
	}



/*****************************************************************************
*
*	FIN methodes statiques
*
******************************************************************************/






	/**
	 * set the prenom
	 * @param the prenom as String
	 */
	public void setPrenom(String prenom) {
		m_prenom = prenom;
	}

	/**
	 * get the nom
	 * @return the nom as String
	 */
	public String getNom() {
		return m_nom;
	}

	/**
	 * set the nom
	 * @param the nom as String
	 */
	public void setNom(String nom) {
		m_nom = nom;
	}





	/**
	 * get the  text
	 * @return the text as a String
	 */
	public String getText() {
		return m_text;
	}

	/**
	 * set the text
	 * @param the text as a String
	 */
	public void setText(String text) {
		m_text = text;
	}

	/**
	 * get the data of the news entry
	 * @retrun the date as a String
	 */
	public String getDate() {
		return date2string(m_date);
	}

	/**
	 * set the data of the news entry
	 * @param the date as a GregorianCalendar date
	 */
	public void setDate(GregorianCalendar date) {
		m_date = date;
	}



	/**
	 * method to access the real name of the channel
	 * @return String     the name of the channel
	 */
	public String getChannel() {
		listesChannelContentDefinition temp = null;
		// create a Object to access the name of the channel
		try{
			temp = new listesChannelContentDefinition(null, new Integer(m_channel));
		}catch(CmsException e) {
			System.err.println("[" + this.getClass().getName() + "] " + e.getMessage());
		}
		return temp.getName();
	}

	/**
	 * set the channel
	 * @param the channel as int
	 */
	public void setChannel(int channel) {
		m_channel = channel;
	}

	/**
	 * method to convert a GregrianCalendar date to a String
	 * @parame the GregrianCalendar date
	 * @return the date as a String
	 */
	public static String date2string(GregorianCalendar cal) {
		DateFormat df;
		df = DateFormat.getDateTimeInstance(DateFormat.MEDIUM,
                                            DateFormat.MEDIUM, m_locale);
		return df.format(cal.getTime());
	}
    /**
	 * method to convert a GregrianCalendar date to a String
	 * @parame the GregrianCalendar date
	 * @return the date as a String
	 */
	public static String date2shortString(GregorianCalendar cal) {
		DateFormat df;
		df = DateFormat.getDateInstance(DateFormat.MEDIUM, m_locale);
		return df.format(cal.getTime());
	}


	/**
	 * method to convert a string date to a GregorianCalendar date.
	 * The method throws a ParseException if the string cannot be converted.
	 * @param the date as a String
	 * @return the date as a GregorianCalendar
	 */
	public static GregorianCalendar string2date(String sDate) throws ParseException {
		DateFormat df;
		GregorianCalendar ret = new GregorianCalendar();
		df = DateFormat.getDateTimeInstance(DateFormat.MEDIUM,
                                            DateFormat.MEDIUM, m_locale);
		java.util.Date date = df.parse(sDate);
		//System.err.println("string2date " + date.toString());
		ret.setTime(date);
		//System.err.println("string2date " + ret.get(Calendar.DATE)+"." + ret.get(Calendar.MONTH)+"." + ret.get(Calendar.YEAR) );
		return ret;
	}

    /**
	 * method to convert a string date to a GregorianCalendar date.
	 * The method throws a ParseException if the string cannot be converted.
	 * @param the date as a String
	 * @return the date as a GregorianCalendar
	 */
	public static GregorianCalendar shortString2date(String sDate) throws ParseException {
		DateFormat df;
		GregorianCalendar ret = new GregorianCalendar();
		df = DateFormat.getDateInstance(DateFormat.MEDIUM, m_locale);
		java.util.Date date = df.parse(sDate);
		//System.err.println("string2date " + date.toString());
		ret.setTime(date);
		//System.err.println("string2date " + ret.get(Calendar.DATE)+"." + ret.get(Calendar.MONTH)+"." + ret.get(Calendar.YEAR) );
		return ret;
	}

	/**
	 * method to convert a string date to a sqlDate date.
	 * The method throws a ParseException if the string cannot be converted.
	 * @param the date as a String
	 * @return the date as a sqlDate
	 */
	public static java.sql.Timestamp string2sqlTimestamp(String sDate) throws ParseException {
		DateFormat df;
		//df = DateFormat.getDateInstance(DateFormat.MEDIUM);
        df = DateFormat.getDateTimeInstance(DateFormat.MEDIUM, DateFormat.MEDIUM, m_locale);
		java.util.Date uDate = df.parse(sDate);
		//System.err.println("string2sqlDate " + uDate.toString());
		java.sql.Timestamp sqlTimestamp = new java.sql.Timestamp(uDate.getTime());  // Date to SQLDate
		return sqlTimestamp;
	}

    /**
	 * method to convert a string date to a sqlDate date.
	 * The method throws a ParseException if the string cannot be converted.
	 * @param the date as a String
	 * @return the date as a sqlDate
	 */
	public static java.sql.Timestamp shortString2sqlTimestamp(String sDate) throws ParseException {
		DateFormat df;
		//df = DateFormat.getDateInstance(DateFormat.MEDIUM);
        df = DateFormat.getDateInstance(DateFormat.MEDIUM, m_locale);
		java.util.Date uDate = df.parse(sDate);
		//System.err.println("string2sqlDate " + uDate.toString());
		java.sql.Timestamp sqlTimestamp = new java.sql.Timestamp(uDate.getTime());  // Date to SQLDate
		return sqlTimestamp;
	}

    public static java.sql.Timestamp date2Timestamp(GregorianCalendar gDate) {
        java.util.Date date= gDate.getTime();
        return new java.sql.Timestamp( date.getTime() );
    }

  /**
   * method to update the poolname
   * is invoked from listesChannelContentDefinition
   * @param String pool
   */
  public static void updatePoolName(String pool) {
    c_pool = pool;
  }
 
  /**
  * Simple enveloppe
  */
  public static boolean checkEmail(String email) {
	return MailUtils.checkEmail(email);
  } 


  /**
   * method to clear the cache
   * is invoked from write and delete
   * @param CmsObject cms
   */

  private void clearcache(CmsObject cms) {

    // first the element cache
    cms.getOnlineElementCache().clearCache();

    URL u = null;
    String p = ((HttpServletRequest)
                cms.getRequestContext().getRequest().getOriginalRequest()
                ).getServletPath();
    String s = ((HttpServletRequest)
                cms.getRequestContext().getRequest().getOriginalRequest()
                ).getServerName();
    try {
        u = new URL("http://" +s +p
                    +"/system/workplace/action/login.html?_clearcache=all");
        URLConnection uc = u.openConnection();
        uc.connect();
        InputStream in = uc.getInputStream();
        //System.err.println("clearcache o.k.!");
    } catch(MalformedURLException e) {
        System.err.println("Exception1: " +e.getMessage() );
    } catch (IOException e) {
        System.err.println("Exception2: " +e.getMessage() );
    }

  }



}

