package com.opencms.modules.listes;
import com.opencms.file.*;
import com.opencms.core.*;
import com.opencms.util.*;
import com.opencms.template.*;
import com.opencms.workplace.*;
import com.opencms.defaults.*;

import java.util.*;
import java.io.*;
import java.sql.*;
import java.text.*;

/**
 * Cette classe controle le template backoffice permettant de crer des batch (threads)
 * Elle prsente un exemple de gestion des options tendues (requiert la version 2.01 de la classe A_CmsBackoffice)
 * 
 * Version 1.0
 * Date 04/02/03
 */
public class listesThreadsBackoffice extends A_CmsBackoffice {

	/* une entre  comprend les informations suivantes :
		Id	
		channelId
		date		
		sujet		
		message		
		etat		
		rapport
		lockstate
	*/
	
	/* champs de formulaire */

	final static String 	TH_CHANNEL	=	"channelName";
	final static String 	TH_DATE		=	"date";
	final static String 	TH_SUJET	=	"sujet";
	final static String 	TH_MESSAGE	=	"message";




	/**
	 */
	public Class getContentDefinitionClass() {
		return listesThreadsContentDefinition.class;
	}

  	/**
   	* when clicking directly on one entry in the list ...
   	*/


	/**
	* Returns the URL of the dialog to add a new data entry.
	 */
	public String getCreateUrl(CmsObject cms, String tagcontent,
                                A_CmsXmlContent doc, Object userObject)
	throws Exception{
		return "system/modules/com.opencms.modules.listes/"
                + "administration/threads/EditBackoffice.html";
	}



	public String getBackofficeUrl(CmsObject cms, String tagcontent,
                                    A_CmsXmlContent doc, Object userObject)
	throws Exception {
		return "system/modules/com.opencms.modules.listes/"+
                "administration/threads/Backoffice.html";
	}

	/**
	* Returns the URL of the dialog to edit an existing data entry.
	 */

	public String getEditUrl(CmsObject cms, String tagcontent,
                             A_CmsXmlContent doc, Object userObject)
	throws Exception {
		return "system/modules/com.opencms.modules.listes/" +
               "administration/threads/EditBackoffice.html";
	}

	/* ============================================================================================================ 
	*
	*  Les fonctions publiques ci-dessous permettent de rajouter des options (2) au menu droulant d'un entre
	*  A chaque mthode publique de type getAddUrlx doit correspondre une mthode de gestion du contenu 
	*  getContentAddUrlx	et une mthode getUrlxLabel permettant de rcuprer le libell associ  l'option 
	*  dans le menu contextuel.
	*  
	*  ATTENTION : l'utilisation de ces mthodes requiert la version 2.01 de la classe :
			com.opencms.defaults.A_CmsBackOffice ainsi que de la nouvelle version du template :
			/system/workplace/templates.Backoffice
	*	
	* 		Ces fonctionnalits sont actuellement en phase de test et ne seront pas implementes 
	*		dans la version courant
	*  ============================================================================================================
	*/

	/*
	
	public String getAddUrl1(CmsObject cms, String tagcontent, A_CmsXmlContent doc, Object userObject)
		throws Exception {
	
		return "system/modules/com.opencms.modules.listes/administration/threads/EditBackoffice.html";
	}
	
	public String getAddUrl2(CmsObject cms, String tagcontent, A_CmsXmlContent doc, Object userObject)
	throws Exception {
		return "spcifiez le chemin relatif";
	}

 	public boolean hasExtendedOptions () {
		return true;
  	}

	public static String getUrl1Label() {

		return "Exemple ";
	}
	
	public static String getUrl2Label() {
		return "";

	}
	*/
	


	// Cette fonction est prpose  la modification d'un dataset existant
	public byte[] getContentEdit(CmsObject cms,
					CmsXmlWpTemplateFile template,
					String elementName,
					Hashtable parameters,
					String templateSelector)
	throws CmsException {

		boolean dateError = false;

  
   		String error = "";
		GregorianCalendar actDate = new GregorianCalendar();
		actDate.add(Calendar.MINUTE, 15); // + 15 min

		// session will be created or fetched
		I_CmsSession session =
                (CmsSession)cms.getRequestContext().getSession(true);
		// get value of hidden input field action
		String action = (String) parameters.get("action");
        	if(action == null) action = "";
		//get value of id
		String id = (String) parameters.get("id");
		if (id == null) id = "";


		// set existing data in the content definition object
		//& go to the done section of the template
    		if( action.equals("save") || action.equals("saveexit")) {
			//move to the done section in the template
			templateSelector = "done";

			//get data from the template
			
			int idIntValue2 = Integer.valueOf(id).intValue();
			String channelName 	= (String) parameters.get(TH_CHANNEL);
			String sDate		= (String) parameters.get(TH_DATE);			
			String sujet		= (String) parameters.get(TH_SUJET);
			String message		= (String) parameters.get(TH_MESSAGE);
						

			// create an Object to access the Id (l'objet channel fait rfrence au nom de la liste)
			listesChannelContentDefinition temp =
            			new listesChannelContentDefinition(channelName);
			int channelId = temp.getIntId();
			GregorianCalendar date = null;
			
			try{
				date = listesThreadsContentDefinition.string2date(sDate);
				// doit vrifier si l'venement se droule dans le pass ou maintenant
				if ( 	listesThreadsContentDefinition.date2Timestamp(date).before(  
					listesThreadsContentDefinition.date2Timestamp(actDate)) ||
					listesThreadsContentDefinition.date2Timestamp(date).equals(
					listesThreadsContentDefinition.date2Timestamp(actDate)) 
				)
					dateError = true;
			}catch(ParseException e) {
				System.err.println("[" + this.getClass().getName() + "] "
                        + e.getMessage());
				dateError = true;
			}


			// listesThreadsContentDefinition editCD =	new listesThreadsContentDefinition(cms, new  Integer(idIntValue2));

			listesThreadsContentDefinition editCD = null;
			try {
				editCD = new listesThreadsContentDefinition(cms, new  Integer(idIntValue2));
			}
			catch (Exception ex) {
				A_OpenCms.log("opencms_info", "listesThreadsContentDefinition.getContentEdit :: "+ex.toString());
			}


			// Vrifie la saisie
			if((sujet.equals("") || message.equals("") || 
				sujet == null || message == null || dateError == true)) {

				templateSelector = "default";
				template.setData(TH_SUJET, sujet);
				template.setData(TH_MESSAGE, message);
				actDate.add(Calendar.MINUTE, 15); // + 15 min !!
                    		template.setData(TH_DATE,listesThreadsContentDefinition.date2string(actDate));	
				// re-display data in selectbox
				session.putValue("checkSelectChannel", editCD.getChannel());
				
				// dtermine l'affichage slctif de l'erreur

				if( sujet.equals("") || sujet == null) {
            				error += template.getProcessedDataValue("missing_sujet", this, parameters) + "<br>";
					template.setData("error", error);
            				template.setData("sujetHighlightStart", template.getData("redStart"));
            				template.setData("sujetHighlightEnd", template.getData("redEnd"));
				}
				if( message.equals("") || message == null) {
            				error += template.getProcessedDataValue("missing_message", this, parameters) + "<br>";
					template.setData("error", error);
            				template.setData("messageHighlightStart", template.getData("redStart"));
            				template.setData("messageHighlightEnd", template.getData("redEnd"));
				}

				if(dateError == true) {
            				error += template.getProcessedDataValue("missing_date", this, parameters) +"<br>";
            				template.setData("dateHighlightStart", template.getData("redStart"));
            				template.setData("dateHighlightEnd", template.getData("redEnd"));
					template.setData("error", error);
				}
				template.setData("setaction", "save");   // still not first time shown
				return startProcessing(cms, template, elementName, parameters, templateSelector);
			}
			

			//change the cd object content
			editCD.setSujet(sujet);
			editCD.setMessage(message);
			editCD.setDate(date);
			editCD.setChannel(channelId);


			try {
				editCD.write(cms);
			} catch (Exception e) {
				System.err.println("listesThreadsEditBackoffice: couldnt update!");
				System.err.println("[" + this.getClass().getName() + "] " + e.getMessage());
				template.setData("error", e.toString());
				template.setData("sujet", sujet);
				template.setData("message",message);
				actDate.add(Calendar.MINUTE, 15); // + 15 min !!
				template.setData("date", listesThreadsContentDefinition.date2string(actDate));
				template.setData("setaction", "save");
				templateSelector = "default";
				return startProcessing(cms, template, elementName, parameters, templateSelector);
			}
		}
		//START: first time here and got valid id parameter:
		//		 Fill the template with data!

        	if(id != null && !id.equals("")) {
				//get data from cd object
				int idIntValue = Integer.valueOf(id).intValue();
				
				
				
				// listesThreadsContentDefinition editCD = new listesThreadsContentDefinition(cms, new  Integer(idIntValue));
				listesThreadsContentDefinition editCD = null;
				try {
					editCD = new listesThreadsContentDefinition(cms, new  Integer(idIntValue));
				}
				catch (Exception ex) {
					A_OpenCms.log("opencms_info", "listesThreadsContentDefinition.getContentEdit :: "+ex.toString());
					
				}



				//set the data from the CD in the template
				template.setData(TH_SUJET, editCD.getSujet());
				
				
				// fonction inutilise : elle tronque le corps du message pour n'en renvoyer que 30 caracres
				// template.setData(TH_MESSAGE, editCD.getMessage());
				template.setData(TH_MESSAGE, editCD.getCompleteMessage());
				
				
				template.setData(TH_DATE, editCD.getDate());
                		template.setData("setaction", "save");
				// re-display data in selectbox
				session.putValue("checkSelectChannel", editCD.getChannel());
			return startProcessing(cms, template, elementName, parameters, templateSelector);
		}
	

	//finally start the processing
	return startProcessing(cms, template, elementName, parameters, templateSelector);
	}


	// Cette fonction est prpose  la cration d'un dataset

	public byte[] getContentNew(CmsObject cms,
					CmsXmlWpTemplateFile template,
					String elementName,
					Hashtable parameters,
					String templateSelector)
	throws CmsException {
		// session will be created or fetched

		I_CmsSession session = (CmsSession) cms.getRequestContext().getSession(true);
		// get value of hidden input field action
		String action = (String) parameters.get("action");
    		if ( action == null) action = "";
		GregorianCalendar actDate = new GregorianCalendar();  // the actual Date
		actDate.add(Calendar.MINUTE, 15); // + 15 min

		boolean dateError = false;   // could the date-string be parsed?
		boolean channelError = false;
		int channelId = 0;
        	String error = "";

		//get value of id and the marker parameters.get("id");
		String id = (String) parameters.get("id");

		if (id == null) id = "";

		//no button pressed: go to the default section!
		if (action == null || action.equals("")) {
			templateSelector = "default";
			template.setData("setaction", "default");
            		// set actual Date
			template.setData("date", listesThreadsContentDefinition.date2string(actDate));
            		String sujet 	= "";
            		String message 	= "";
 
			//store marker in the session
			//confirmation button pressed, process data!
		} else {
			//create new data, therefore create new content definition instance
			//& go to the done section of the template
			templateSelector = "done";

			//read value of the inputfields
			String sujet 		= (String) parameters.get(TH_SUJET);
			String message		= (String) parameters.get(TH_MESSAGE);
			
			String sDate		= (String) parameters.get(TH_DATE);
			String channelName 	= (String)parameters.get(TH_CHANNEL);

			if(channelName == null || channelName.equals(" ") || channelName.equals("") ) {
				channelError = true;
			}else {
				// create an ChannelObject to access the Id
				listesChannelContentDefinition temp = new listesChannelContentDefinition(channelName);
				channelId = temp.getIntId();
			}
			GregorianCalendar date = null;
			
			try{
				date = listesThreadsContentDefinition.string2date(sDate);
				// doit vrifier si l'venement se droule dans le pass ou maintenant
				if ( 	listesThreadsContentDefinition.date2Timestamp(date).before(  
					listesThreadsContentDefinition.date2Timestamp(actDate)) ||
					listesThreadsContentDefinition.date2Timestamp(date).equals(
					listesThreadsContentDefinition.date2Timestamp(actDate)) 
				) {
					date = actDate;		// positionne dclanchement au moins 15 aprs
					
				}
			}catch(ParseException e) {
				System.err.println("getContentNew: ParseException");
				System.err.println("[" + this.getClass().getName() + "] " + e.getMessage());
				dateError = true;
			}
			// ensure something was filled in!
			if(	sujet.equals("") || sujet == null ||  
				message.equals("") || message == null ||
				dateError == true || channelError == true) {

				templateSelector = "default";
				template.setData(TH_SUJET, sujet);
				template.setData(TH_MESSAGE, message);
				template.setData(TH_DATE,sDate);
				actDate.add(Calendar.MINUTE, 15); // + 15 min !!
                    		listesThreadsContentDefinition.date2string(actDate);


				// re-display data in selectbox
				session.putValue("checkSelectChannel", ""+channelId);
				
				// dtermine l'affichage slctif de l'erreur

				if( sujet.equals("") || sujet == null) {
            				error += template.getProcessedDataValue("missing_sujet", this, parameters) + "<br>";
					template.setData("error", error);
            				template.setData("sujetHighlightStart", template.getData("redStart"));
            				template.setData("sujetHighlightEnd", template.getData("redEnd"));
				}
				if( message.equals("") || message == null) {
            				error += template.getProcessedDataValue("missing_message", this, parameters) + "<br>";
					template.setData("error", error);
            				template.setData("messageHighlightStart", template.getData("redStart"));
            				template.setData("messageHighlightEnd", template.getData("redEnd"));
				}

				if(dateError == true) {
            				error += template.getProcessedDataValue("missing_date", this, parameters) +"<br>";					
            				template.setData("dateHighlightStart", template.getData("redStart"));
            				template.setData("dateHighlightEnd", template.getData("redEnd"));
					template.setData("error", error);
				}
				template.setData("setaction", "save");   // still not first time shown
				return startProcessing(cms, template, elementName, parameters, templateSelector);
			}

			

			//create new cd object with the fetched values of the inputfields
			listesThreadsContentDefinition newCD = new listesThreadsContentDefinition(
				channelId,
				date,
				sujet,
				message,
				0,
				"");

			try {
				newCD.write(cms);
			} catch (Exception e) {
				System.err.println("listesThreadsInsertBackoffice: couldnt write!");
				System.err.println("[" + this.getClass().getName() + "] " + e.getMessage());
				template.setData("error", e.toString());
				template.setData(TH_SUJET, sujet);
				template.setData(TH_MESSAGE, message);
				actDate.add(Calendar.MINUTE, 30); // + 30 min !!
                		template.setData(TH_DATE, listesThreadsContentDefinition.date2string(actDate));
				template.setData("setaction", "save");
				templateSelector = "default";
			}
		}

		//finally start the processing
		return startProcessing(cms, template, elementName, parameters, templateSelector);
	}


   
	/* Fonction de gestion associe  l'url additionnelle 1. Ceci est un squelette d'example
	 * l'id de l'objet (dans ce cas l'id de la liste de diffusion) est dans l'argument id.
	 */
	
	

  	public byte[] getContentAddUrl1(CmsObject cms,
					CmsXmlWpTemplateFile template, 
					String elementName, Hashtable parameters, 
					String templateSelector) 

	throws CmsException {


    		return startProcessing(cms, template, elementName, parameters, templateSelector);
  	}

		


}

