package com.opencms.modules.listes;

import com.opencms.core.*;
import com.opencms.file.*;
import com.opencms.template.*;
import com.opencms.defaults.*;
import com.opencms.file.mySql.*;
import com.opencms.util.*;
import java.util.*;
import java.text.*;
import java.io.*;
import java.sql.*;
import java.net.*;               // URL

import javax.servlet.http.*;     // HttpServletRequest

/**
 * This is the Content Definition class for the listesThreads entries.
 * The class privides some constructors filter methods to access entries as well
 * as some methods to convert dates and strings.
 * Version 1.0
 * Date 04/02/03

 */
public class listesThreadsContentDefinition extends A_CmsContentDefinition implements I_CmsContent {

	// constants for day or month selection
	private static final Integer C_FLAG_DAY = new Integer(0);
	private static final Integer C_FLAG_MONTH = new Integer(1);

	// Une entre de thread. Il faudra respecter l'ordre des champs

	private int m_id 					= -1;		//	1
	private int m_channel 					= -1;		//	2
	private GregorianCalendar m_date 			= null;		//	3
	private String m_sujet			 		= "";		//	4
	private String m_message	 			= "";		//	5
	private int    m_etat		 			= 0;		//	6
	private String m_rapport				= "";		//	7
	private int m_lockstate 				= -1;		//	8
	
    private static Locale m_locale = new Locale("Default", ""+Locale.getDefault());
	/*
	*				0	mis en queue
	*				1	en cours d'envoi
	*				2	envoy	
	*				3	problme d'envoi (consulter le rapport)		
	*/


    private final static String C_STATES[] =  {	"mis en queue",
							"en cours d'envoi",
							"envoy",
							"erreur" };
        						
	

	private boolean newElement = false;  // indicates if an element should be written or updated

	private static String c_pool = null;
	// private static NewsKeyAccess m_NewsKeyAccess = null;

	// Constants for table news_entry
	private static final String  C_SELECT_ALL = "select * from listes_threads";
	private static final String  C_INSERT_ENTRY = "insert into listes_threads values(?,?,?,?,?,?,?,?)";
	private static final String  C_DELETE_ENTRY = "delete from listes_threads where ID = ?";
	private static final String  C_COUNT = "select count(*) from listes_threads";
    private static final String  C_SELECT_ID = "select * from listes_threads where ID = ?";
	private static final String  C_GET_ALL = "select * from listes_threads";
	private static final String  C_UPDATE_ENTRY = "update listes_threads set  channel = ?,date = ?, sujet = ?, message = ?, etat = ?, rapport =?, lockstate = ? where id = ?";
	private static final String  C_ORDER = "select * from listes_threads order by ? desc";
	private static final String  C_SELECT_CHANNEL_ID = "select count(*) from listes_threads where channel = ?";
	private static final String  C_SELECT_MONTH = "select * from listes_threads where month(date)= ? and channel = ? order by date desc";
	private static final String  C_SELECT_DAY = "select * from listes_threads where (date between ? and ?) and (channel = ?)";
	private static final String  C_SELECT_FIRST_N_OF_CHANNEL = "select * from listes_threads where channel = ? order by date desc";
	private static final String  C_SELECT_BY_STATUS = "select * from listes_threads where etat = ?";

	private static final String  C_UPDATE_ETAT = "update listes_threads set  etat = ? where id = ?";
	private static final String  C_UPDATE_RAPPORT = "update listes_threads set  rapport = ? where id = ?";


	public static final String	C_TABLE_LISTETHREADS = "LISTES_THREADS";
	public static final int      	C_NOT_LOCKED = -1;



 	/* This code is executed on load-time of this class */
  	static {
    		try {
      			// set the pool
      			moduleParameterWasUpdated(null);
    		} catch(CmsException  exc) {
      			// TODO: Error-handling
    		}
  	}

  	/**
   	* This event-method is invoked if a user changes a parameter for a module in the module-
   	* administration.
   	* It reads the current name of the pool for this module and stores it to c_pool
   	* @param CmsObject - the cmsobject to get access to the cms.
   	*/
  	public static void moduleParameterWasUpdated(CmsObject cms) throws CmsException {    	
		c_pool = OpenCms.getRegistry().getModuleParameterString("com.opencms.modules.listes", "poolname");
		listesAboContentDefinition.updatePoolName(c_pool);
  	}


	/**
	 * Constructor
	 */
	public listesThreadsContentDefinition() {
		super();
	}

	/**
	 * Pour lire un enregistrement, il est ncessaire d'instancier ce constructeur
	 * is needed by A_Backoffice like this!
	 */
	public listesThreadsContentDefinition(CmsObject cms, Integer ID) throws Exception {
		java.util.Date date = null;
		m_date = new GregorianCalendar();
		
    	Connection con = null;
		PreparedStatement statement = null;
		ResultSet res = null;
		try {
      		con = DriverManager.getConnection(c_pool);
		 	statement = con.prepareStatement(C_SELECT_ID);
			statement.setInt(1, ID.intValue());  // choose id of the Element  "where id == "
			res = statement.executeQuery();
			if(res.next()){
				// read the items from the resultset
				m_id 		= res.getInt(1);

				// System.err.println("NewsContentDefinition" + m_date.get(Calendar.DATE)+"." + m_date.get(Calendar.MONTH)+"." + m_date.get(Calendar.YEAR) );
				m_channel 	= res.getInt(2);

               	java.sql.Timestamp timestamp 	= res.getTimestamp(3);
				java.util.Date tempDate 	= (java.util.Date)timestamp;
				m_date.setTime( tempDate ); 	// SQLDate to Date to GregorianCalendar

				m_sujet 	= res.getString(4);
				m_message 	= res.getString(5);
				m_etat 		= res.getInt(6);
				m_rapport	= res.getString(7);

				m_lockstate 	= res.getInt(8);
			}
		}
		catch(SQLException e) {
			A_OpenCms.log("opencms_info", "listesThreadsContentDefinition.listesThreadsContentDefinition :: "+e.toString());
		}
		finally{
			try{
				res.close();
			}catch (Exception e) {
				// ignore the exception
			}
            		try{
				statement.close();
			}catch (Exception e) {
				// ignore the exception
			}
            		try{
				con.close();
			}catch (Exception e) {
				// ignore the exception
			}
		}
	}

	/**
	 * Constructor to create an Element
	 *
	 */
	public listesThreadsContentDefinition(
			int 			channel,
			GregorianCalendar 	date,
			String 			sujet,
			String 			message,
			int			etat,
			String			rapport)
	{
		
		m_channel	= channel;
		m_date		= date;
		m_sujet		= sujet;
		m_message	= message;
		m_etat		= etat;
		m_rapport	= rapport;
		m_lockstate	= C_NOT_LOCKED; // default
		newElement 	= true;        	// this is a new Element
		try{
			m_id = com.opencms.dbpool.CmsIdGenerator.nextId(c_pool, C_TABLE_LISTETHREADS);
		} catch(CmsException e){
			
			A_OpenCms.log("opencms_info", "listesThreadsContentDefinition.listesThreadsContentDefinition :: "+e.toString());
		}
	}

	/**
	 * Constructor to create an Element with a certain ID
	 * needed to create a list of Elementes
	 *
	 * @param id    The unique id
	 * @param name  The name
	 * @param descr The descr
	 */
	private listesThreadsContentDefinition(	int id,
						int channel,
						java.sql.Timestamp date,
						String sujet,
						String message,
						int etat,
						String rapport,
						int lockstate) {
		m_id		= id;
		m_channel	= channel;
		m_sujet		= sujet;
		m_message	= message;
		m_etat		= etat;
		m_rapport	= rapport;
		m_date 		= new GregorianCalendar();
        	java.util.Date tmpDate = new java.util.Date(date.getTime());
	
        	m_date.setTime(tmpDate);
		m_lockstate = lockstate;
	}


	/**
	 * write an Element to the database
	 * @param cms An instance of CmsObject
	 */
	public void write(CmsObject cms) throws Exception{
        Connection con = null;
		PreparedStatement statement = null;
		ResultSet res = null;
		java.util.Date uDate = m_date.getTime();  	// GregorianCalendar to Date
        	//clearcache(cms); // clear the cache to make changes visible
		
        	java.sql.Timestamp sqlDate = date2Timestamp(m_date);
        
		if(newElement == true) {
			// Element not jet in Database -> insert Statement
			try {
				// write the data to database
                		con = DriverManager.getConnection(c_pool);
		 		statement = con.prepareStatement(C_INSERT_ENTRY);
				statement.setInt(1, m_id);
				statement.setInt(2, m_channel);
				statement.setTimestamp(3, sqlDate);
				statement.setString(4, m_sujet);
				statement.setString(5, m_message);
				statement.setInt(6, m_etat);				
				statement.setString(7,m_rapport);
                		statement.setInt(8, m_lockstate);

				statement.execute();
				newElement = false; // element is no longer new!
			}
			catch(Exception e) {
				A_OpenCms.log("opencms_info", "listesThreadsContentDefinition.write :: "+e.toString());
			}
			finally{
                		try{
				    statement.close();
			    	}catch (Exception e) {
                    			// ignore the exception
			    	}
                		try{
                    			con.close();
				}catch (Exception e) {
				    // ignore the exception
				}
			}

		}else {
			// Element is already in Database -> update statement
			update(cms, sqlDate);
		}
	}



	/**
	 * Insertion d'un lment en base de donnes (les donnes sauvegardes sont passe en paramtre)
	 * @param cms : CmsObject
	 * @param => valeurs stockes en base de donnes...
	 */
	public void write(CmsObject cms, int id, int channel, GregorianCalendar gDate, String sujet, String message, int etat, String rapport, int lockstate ) throws Exception{
        Connection con = null;
		PreparedStatement statement = null;
		ResultSet res = null;
		java.util.Date uDate = m_date.getTime();  	// GregorianCalendar to Date
		
        java.sql.Timestamp sqlDate = date2Timestamp(gDate) ;
        
		if(newElement == true) {
			try {
				// insertion en base de donnes
                con = DriverManager.getConnection(c_pool);
		 		statement = con.prepareStatement(C_INSERT_ENTRY);
				statement.setInt(1, id);
				statement.setInt(2, channel);
				statement.setTimestamp(3, sqlDate);
				statement.setString(4, sujet);
				statement.setString(5, message);
				statement.setInt(6, etat);				
				statement.setString(7, rapport);
          		statement.setInt(8, lockstate);

				statement.execute();
				newElement = false; // element is no longer new!
			}
			catch(Exception e) {
				A_OpenCms.log("opencms_info", "listesThreadsContentDefinition.write :: "+e.toString());
			}
			finally{
                try{
				    statement.close();
			    }
			    catch (Exception e) {
                  	A_OpenCms.log("opencms_info", "listesThreadsContentDefinition.write :: Erreur survene lors de la cloture de l'objet statement "+e.toString());
			    }
                try{
                	con.close();
				}
				catch (Exception e) {
                 	A_OpenCms.log("opencms_info", "listesThreadsContentDefinition.write :: Erreur survene lors de la cloture de la connexion "+e.toString());
				}
			}

		}
		else {
			// Element is already in Database -> update statement
			update(cms, sqlDate);
		}
	}



	/**
	 * method to updata an existing entry.
	 * This method is only called by the write()-method.
	 * @param CmsObject
	 * @param the new date (java.sql.Date)
	 */
	private void update(CmsObject cms, java.sql.Timestamp sqlDate)  {
        Connection con = null;
		PreparedStatement statement = null;
		ResultSet res = null;
		try {
			// write the data to database
            con = DriverManager.getConnection(c_pool);

		 	statement = con.prepareStatement(C_UPDATE_ENTRY);

			statement.setInt(1, m_channel);
			statement.setTimestamp(2, sqlDate);
			statement.setString(3, m_sujet);
			statement.setString(4, m_message);
			statement.setInt(5, m_etat);				
			statement.setString(6,m_rapport);
            statement.setInt(7, m_lockstate);
			statement.setInt(8, m_id);

			statement.execute();

		}catch(Exception e) {
			A_OpenCms.log("opencms_info", "listesThreadsContentDefinition.update :: "+e.toString());
		}finally{
            		try{
				statement.close();
			}catch (Exception e) {
				// ignore the exception
			}
            		try{
				con.close();
			}catch (Exception e) {
				// ignore the exception
			}
		}
	}

	/**
	 * delete an Element from the database
	 * TOTO : effectuer les contrles d'intgrit rfrentielle
	 * @param cms An instance of CmsObject
	 */
	public void delete(CmsObject cms)  {
        	Connection con = null;
		PreparedStatement statement = null;
		ResultSet res = null;
        	//clearcache(cms); // clear the cache to make changes visible
		try {
			// delet data from database
            		con = DriverManager.getConnection(c_pool);
		 	statement = con.prepareStatement(C_DELETE_ENTRY);
			statement.setInt(1, m_id);
			statement.execute();
		}
		catch(Exception e) {
			A_OpenCms.log("opencms_info", "listesThreadsContentDefinition.delete :: "+e.toString());
		}
		finally{
			 try{
				statement.close();
			}catch (Exception e) {
				// ignore the exception
			}
            		try{
				con.close();
			}catch (Exception e) {
				// ignore the exception
			}
		}
	}

  	/**
	 * create a pool if necessary
	 *
	 * @return c_pool
	 */
	private static String getPoolName(CmsObject cms) {

		if(c_pool == null) {
      			try{
        			c_pool = cms.getRegistry().getModuleParameterString("com.opencms.modules.listes", "poolname");
      			} catch(CmsException exc) {
        			// TODO: Add error-handling here!
      			}
		}
		return c_pool;
	}


	/**
	*	The three methods getFieldNames(), getFieldMethods() and getFilterMethods() are used
		by the Backoffice classes to generate the lists of entries.
	*
	**/



	/**
	 * declare the entries that should be displayed in the Backoffice list
	 * @return a Vector
	 */
	public static Vector getFieldMethods(CmsObject cms) {
		Vector methods = new Vector();
		try {
			methods.addElement(listesThreadsContentDefinition.class.getMethod("getChannel", new Class[0]));
			methods.addElement(listesThreadsContentDefinition.class.getMethod("getDate", new Class[0]));
			methods.addElement(listesThreadsContentDefinition.class.getMethod("getSujet", new Class[0]));
			methods.addElement(listesThreadsContentDefinition.class.getMethod("getMessage", new Class[0]));
			methods.addElement(listesThreadsContentDefinition.class.getMethod("getEtat", new Class[0]));
			methods.addElement(listesThreadsContentDefinition.class.getMethod("getRapport", new Class[0]));
		}
		catch(NoSuchMethodException e) {
			A_OpenCms.log("opencms_info", "listesThreadsContentDefinition.delete :: "+ e.toString() );
		}
		return methods;
	}

	/**
	 * This method returns the labels of the columns for the list of Content Definition entries generated by the Backoffice classes.
	 * @return a Vector
	 */
	public static Vector getFieldNames(CmsObject cms) {
		Vector names = new Vector();
		names.addElement("Channel");
		names.addElement("Date");
		names.addElement("Sujet");
		names.addElement("Message");
		names.addElement("Etat");
		names.addElement("Rapport");
		return names;
	}

	/**
	 * declare the methods to filter the list entries
	 * @return A Vector
	 *
	 */
	public static Vector getFilterMethods(CmsObject cms) {
		Vector filterMethods = new Vector();
        	String showText = "";
		Integer actChannelId = null;  // has to be Integer because of Reflection!
        	Vector channels = null;
        	GregorianCalendar actDate = new GregorianCalendar();
        	int actMonth = actDate.get(Calendar.MONTH) + 1;
    		try {
	  		channels = listesChannelContentDefinition.getChannelList(); // rcupre les listes de diffusion
    		}
    		catch(CmsException e) {
      			return null;
    		}
	  	// add the always shown first filterMethod...
		try {
			filterMethods.addElement(new CmsFilterMethod("Tous les envois tris par ...", listesThreadsContentDefinition.class.getMethod("getSortedList", new Class[] {String.class}), new Object[] {}));
		}catch(NoSuchMethodException e) {
			System.err.println("Exception in listesThreadsContentDefinition.getFilterMethods(CmsObject)!"+ e.getMessage());
		}
		// try to add some filters dynamically
		for(int i=0; i< channels.size(); i++) {
			showText = ((listesChannelContentDefinition)channels.elementAt(i)).getName();
			actChannelId = new Integer(((listesChannelContentDefinition)channels.elementAt(i)).getIntId());
			try {
				filterMethods.addElement(new CmsFilterMethod(showText +": les derniers enregistrements...", listesThreadsContentDefinition.class.getMethod("getList", new Class[] {Integer.class, String.class}), new Object[] {actChannelId}, ""));				
				filterMethods.addElement(new CmsFilterMethod(showText +": Enregistrements du mois : " ,  listesThreadsContentDefinition.class.getMethod("getDynamicList", new Class[] {Integer.class, Integer.class, String.class}), new Object[] {actChannelId, C_FLAG_MONTH}, ""+actMonth));
			}catch(NoSuchMethodException e) {
				A_OpenCms.log("opencms_info", "listesThreadsContentDefinition.getFilterMethods :: "+ e.toString());
			}
		}

		return filterMethods;
	}

	/**
	 * one filter method (par liste)
	 * this method creates a list of all entries
	 *
	 * @return a Vector with all Elementes
	 */
	public static Vector getList(Integer channel, String stringN) {
		Vector list = new Vector();
        	Connection con = null;
		PreparedStatement statement = null;
		ResultSet res = null;
		int i =  0;
		int n = -1;
		
		try{
			n = Integer.parseInt(stringN);
		}catch(NumberFormatException e) {
			//System.err.println("Exception in listesThreadsContentDefinition.getNewsList()!"+e.getMessage());
			n = -1;
		}
		try {
            con = DriverManager.getConnection(c_pool);
		 	statement = con.prepareStatement(C_SELECT_FIRST_N_OF_CHANNEL);
			statement.setInt(1, channel.intValue());
			res = statement.executeQuery();
			while(res.next() && (i != n)){
				// read the item from the resultset
				// insert Element into list
				list.addElement(new listesThreadsContentDefinition(
				res.getInt(1),		// Id
				res.getInt(2),  	// Channel
				res.getTimestamp(3),  	// Date envoi
				res.getString(4),  	// Sujet
				res.getString(5),  	// Corps du message		
				res.getInt(6),  	// Etat
				res.getString(7),	// Rapport 
				res.getInt(8)    	// Lockstate
				));
				i++;
			}
		}catch(Exception e) {
				A_OpenCms.log("opencms_info", "listesThreadsContentDefinition.getList :: "+ e.toString());
		}finally{
			try{
				res.close();
			}catch (Exception e) {
				// ignore the exception
			}
            		try{
				statement.close();
			}catch (Exception e) {
				// ignore the exception
			}
            		try{
				con.close();
			}catch (Exception e) {
				// ignore the exception
			}
		}
		return list;
	}


	/**
	 * one filter method
	 * Permet de trier selon un champ donn (le nom du champs doit correspondre)
	 * TODO : Ne tient pas compte du fichier des langues
	 *
	 * @return a Vector with all Elementes
	 */

	public static Vector getSortedList(String str) {
		Vector list = new Vector();
        	Connection con = null;
		PreparedStatement statement = null;
		ResultSet res = null;
		if(str == null || str == " ") { str = ""; }
		str.trim();
		str.toLowerCase();
		// System.err.println("[listesThreadsContentDefinition.getSortedList()] Start - str: " +str);
		try {
            		con = DriverManager.getConnection(c_pool);
		 	statement = con.prepareStatement(C_ORDER);
			if( str.equals("") ) {
				statement.setString(1, "date");
			}
			else if( str.equals("id") ) {
				statement.setString(1, "id");
			}
			else if(str.equals("channel")  ) {
				statement.setString(1, "channel");
			}
			else if(str.equals("sujet") ) {
				statement.setString(1, "sujet");
			}
			else if(str.equals("etat") || str.equals("tat")) {
				statement.setString(1, "etat");
			}
			else if(str.equals("date") ) {
				statement.setString(1, "date");
			}
			else {
				statement.setString(1, "date");	// all other inputs! par dfaut
			}
			res = statement.executeQuery();
			while(res.next()){
				// read the item from the resultset
				// insert Element into list
				list.addElement(new listesThreadsContentDefinition(
				res.getInt(1),		// Id
				res.getInt(2),  	// Channel
				res.getTimestamp(3),  	// Date envoi
				res.getString(4),  	// Sujet
				res.getString(5),  	// Corps du message		
				res.getInt(6),  	// Etat
				res.getString(7),	// Rapport
				res.getInt(8)    	// Lockstate


				));
			}
		}catch(Exception e) {
				A_OpenCms.log("opencms_info", "listesThreadsContentDefinition.getSortedList :: "+ e.toString());
		}finally{
			try{
				res.close();
			}catch (Exception e) {
				// ignore the exception
			}
            		try{
				statement.close();
			}catch (Exception e) {
				// ignore the exception
			}
            		try{
				con.close();
			}catch (Exception e) {
				// ignore the exception
			}
		}
		return list;
	}

 
	/**
	 * one filter method
	 * @return a Vector with all selected Elementes
	 */
	public static Vector getDynamicList(Integer channelId, Integer flag, String str) throws CmsException {

		Vector list 			= new Vector();
		java.sql.Timestamp day 		= null;
        	GregorianCalendar dayEnd 	= new GregorianCalendar();
        	Connection con 			= null;
		PreparedStatement statement 	= null;
		ResultSet res 			= null;
		int month 			= 1;
		//System.err.println("[listesThreadsContentDefinition.getDynamicList()] Start - channelId:" +""+channelId.intValue() + "flag: " + ""+flag.intValue() +"str: " + str);
		if(flag.equals(C_FLAG_MONTH)) {
			try{
				month = Integer.parseInt(str);
			}catch(NumberFormatException e) {
				A_OpenCms.log("opencms_info", "listesThreadsContentDefinition.getDynamicList :: "+ e.toString());
			}
			if(month <1 || month >12) month = 1;
		}
		else
		{
			// "day"
			if( (str == null) || (str.equals("")) || (str.equals(" "))) {
				try{
					//day = shortString2sqlTimestamp("01.01.2000"); // this default value seems to be needed by A_Backoffice
					day = date2Timestamp(dayEnd);
                    			dayEnd.set(Calendar.HOUR, 23);
                    			dayEnd.set(Calendar.MINUTE, 59);
                    			dayEnd.set(Calendar.SECOND, 59);
                }
                catch(Exception e) {
					A_OpenCms.log("opencms_info", "listesThreadsContentDefinition.getDynamicList :: "+ e.toString());
				}
			}else {
				try{;
					day = shortString2sqlTimestamp(str);  // now a real date is parsed
                    			dayEnd = shortString2date(str);
                    			dayEnd.set(Calendar.HOUR, 23);
                    			dayEnd.set(Calendar.MINUTE, 59);
                    			dayEnd.set(Calendar.SECOND, 59);
				}
				catch(ParseException e) {
					A_OpenCms.log("opencms_info", "listesThreadsContentDefinition.getDynamicList :: "+ e.toString());
				}
			}
		}
			

		try{
            			con = DriverManager.getConnection(c_pool);
				if(flag.equals(C_FLAG_MONTH)){
					statement = con.prepareStatement(C_SELECT_MONTH);
					statement.setInt(1, month);
					statement.setInt(2, channelId.intValue());
				}else{
					// "day"
					statement = con.prepareStatement(C_SELECT_DAY);
					statement.setTimestamp(1, day);
                			statement.setTimestamp(2, date2Timestamp(dayEnd) );
					statement.setInt(3, channelId.intValue());
				}
				res = statement.executeQuery();
				while(res.next()){
					// read the item from the resultset
					// insert Element into list
					list.addElement(new listesThreadsContentDefinition(
						res.getInt(1),
						res.getInt(2),  	// Channel
						res.getTimestamp(3),  	// Date
						res.getString(4),  	// Sujet
						res.getString(5),  	// Message
						res.getInt(6),    	// Etat
						res.getString(7),  	// Rapport
						res.getInt(8)    	// Lockstate
					));

				}
		}catch(Exception e) {
 				A_OpenCms.log("opencms_info", "listesThreadsContentDefinition.getDynamicList :: "+ e.toString());
				//System.err.println("[NewsContentDefinition.getDynamicList() Exception while trying to execute the statement:]" + e.getMessage());
		}finally{
            		try{
				res.close();
			}catch (Exception e) {
				// ignore the exception
			}
			try{
				statement.close();
			}catch (Exception e) {
				// ignore the exception
			}
            		try{
				con.close();
			}catch (Exception e) {
				// ignore the exception
			}
		}
		return list;
	}

	/**
	* Cette mthode permet de rcuprer l'ensemble des thread par tat
	*
	*/


	public static Vector getThreadsByStatus(int etat) {
		Vector list = new Vector();
        	Connection con = null;
		PreparedStatement statement = null;
		ResultSet res = null;
		
		try {
            con = DriverManager.getConnection(c_pool);
		 	statement = con.prepareStatement(C_SELECT_BY_STATUS);
			
			statement.setInt(1, etat);	
			
			res = statement.executeQuery();
			while(res.next()){
				// read the item from the resultset
				// insert Element into list
				list.addElement(new listesThreadsContentDefinition(
				res.getInt(1),		// Id
				res.getInt(2),  	// Channel
				res.getTimestamp(3),  	// Date envoi
				res.getString(4),  	// Sujet
				res.getString(5),  	// Corps du message		
				res.getInt(6),  	// Etat
				res.getString(7),	// Rapport
				res.getInt(8)    	// Lockstate
				));
			}
		}catch(Exception e) {
				A_OpenCms.log("opencms_info", "listesThreadsContentDefinition.getTrheadsByStatus :: "+ e.toString());
		}finally{
			try{
				res.close();
			}catch (Exception e) {
				// ignore the exception
			}
            		try{
				statement.close();
			}catch (Exception e) {
				// ignore the exception
			}
            		try{
				con.close();
			}catch (Exception e) {
				// ignore the exception
			}
		}
		return list;
	}







	/** ************************************************************************************
	*	METHODES GET (Elle sont appeles directement par le Backoffice)
	*
	**  ************************************************************************************/


	/**
	* Mthode pour rcuperer l'tat du thread
	*	Etats possibles :
	*				
	*				0	mis en queue
	*				1	en cours d'envoi
	*				2	envoy	
	*				3	problme d'envoi (consulter le rapport)		
	*/
	public String getEtat() {
		try {
			return C_STATES[m_etat];
		}
		catch(ArrayIndexOutOfBoundsException ex) {
			return "inconnu";
		}
	}


	/**
	 * method to access the real name of the channel (nom de liste)
	 * @return String     the name of list
	 */
	public String getChannel() {
		listesChannelContentDefinition temp = null;
		// create a Object to access the name of the channel
		try{
			temp = new listesChannelContentDefinition(null, new Integer(m_channel));
		}catch(CmsException e) {
			A_OpenCms.log("opencms_info", "listesThreadsContentDefinition.getChannel :: "+ e.toString());
		}
		return temp.getName();
	}


	/**
	 * methode pour acceder a l'objet listesChannelContentDefinition
	 * @return String the name of list
	 */
	public listesChannelContentDefinition getChannelObject() {
		listesChannelContentDefinition temp = null;
		// create a Object to access the name of the channel
		try{
			temp = new listesChannelContentDefinition(null, new Integer(m_channel));
		}catch(CmsException e) {
			A_OpenCms.log("opencms_info", "listesThreadsContentDefinition.getChannelObject :: "+ e.toString());
		}
		return temp;
	}


	/**
	 * method to access the real name of the channel (nom de liste)
	 * @return String     the name of list
	 */
	public String getStrChannel() {
		listesChannelContentDefinition temp = null;
		// create a Object to access the name of the channel
		try{
			temp = new listesChannelContentDefinition(null, new Integer(m_channel));
		}catch(CmsException e) {
			A_OpenCms.log("opencms_info", "listesThreadsContentDefinition.getStrChannel :: "+ e.toString());
		}
		return temp.getName();
	}

	/**
	 * get the data of the thread entry
	 * @retrun the date as a String
	 */
	public String getDate() {
		return date2string(m_date);
	}

	/**
	 * renvoit le corps du message (30 premiers octets)
	 * @return as String
	 */
	public String getMessage() {
		String strVide = new String("Affichage a l'edition du message");
		
		// return m_message.substring(0,(m_message.length() >30)?30:m_message.length());
		return strVide;
	
	}


	/**
	 * renvoit le corps COMLPET du message (Tous les caractres)
	 * @return as String
	 */
	public String getCompleteMessage() {
		return m_message;
	}



	/**
	 * get the Unique Id as int
	 * @return m_id (as int)
	 */
	public int getIntId() {
		return m_id;
	}

	/**
	 * renvoit le sujet
	 * @return String
	 */
	public String getSujet() {
		return m_sujet;
	}



	public String getRapport() {
		return m_rapport;
	}

	/**
	 * gets the lockstate of the CD
	* (possibles states: locked by user,locked by other, unlocked)
	*/
	public int getLockstate() {
		return m_lockstate;
	}


	/**
	 * get the unique Id (for abstract Backoffice class)
	 * @return m_id The unique id (as a String)
	 */
	public String getUniqueId(CmsObject cms) {
		return ""+m_id;
	}



	/** ************************************************************************************
	*	METHODES SET (Elle sont appeles directement par le Backoffice)
	*
	**  ************************************************************************************/




	/**
	 * sets the lockstate of the CD.
	 * @param lockstate The lockstate of this CD object.
	 * (possibles states: locked by you,locked by other, unlocked)
	 */
	public void setLockstate(int lockstate) {
		m_lockstate = lockstate;
	}




	/**
	 * initialise le champ Sujet
	 * @param  as String
	 */
	public void setSujet(String sujet) {
		m_sujet = sujet;
	}



	/**
	 * initialise le champ message
	 * @param as String
	 */
	public void setMessage(String message) {
		m_message = message;
	}





	/**
	 * set the data of the news entry
	 * @param the date as a GregorianCalendar date
	 */
	public void setDate(GregorianCalendar date) {
		m_date = date;
	}





	/**
	 * set the channel
	 * @param the channel as int
	 */
	public void setChannel(int channel) {
		m_channel = channel;
	}


	/**
	*	Attention, il ne faut pas que l'tat soit > 3
	*
	*/

	public void setEtat(int etat) {
		m_etat = etat;
	}


	public void setRapport(String rapport) {
		m_rapport = rapport;

	}


	/**
	* if the content should be lockable
	* the method returns true otherwise false
	* @returns a boolean
	*/
	public static boolean isLockable() {
		return false;
	}


	/** ************************************************************************************
	*	MISE A JOUR EN BASE DE DONNEES
	*
	**  ************************************************************************************/

	/**
	 * @param int : l'identifiant du Thread courant
	 * @param int : le nouvel etat du Thread courant
	 * @param out : void
	 */
	public void updateEtat (int id, int etat) throws CmsException {
        Connection con = null;
		PreparedStatement stat = null;
		ResultSet res = null;
		try {

			// write the data to database
            con = DriverManager.getConnection(c_pool);

		 	stat = con.prepareStatement(C_UPDATE_ETAT);
			
			stat.setInt(1, m_etat);				
			stat.setInt(2, m_id);

			res = stat.executeQuery();
			boolean b = res.rowUpdated();

			// statement.execute();

		}
		catch(Exception e) {
			A_OpenCms.log("opencms_info", "listesThreadsContentDefinition.updateEtat :: "+ e.toString());
		}finally{
            try{
            	res.close();
				// statement.close();
			}catch (Exception e) {
				// ignore the exception
			}
            try{
				con.close();
			}
			catch (Exception e) {
				// ignore the exception
			}
		}
	}


	/**
	 * @param int : l'identifiant du Thread courant
	 * @param int : le nouvel etat du Thread courant
	 * @param out : void
	 */
	
	public void updateRapport (int id, String rapport) throws CmsException {
        Connection con = null;
		PreparedStatement stat = null;
		ResultSet res = null;
		try {
			// write the data to database
            con = DriverManager.getConnection(c_pool);

		 	stat = con.prepareStatement(C_UPDATE_RAPPORT);
			
			stat.setString(1, m_rapport);				
			stat.setInt(2, m_id);

			res = stat.executeQuery();
			boolean b = res.rowUpdated();

			// statement.execute();
		}
		catch(Exception e) {
			A_OpenCms.log("opencms_info", "listesThreadsContentDefinition.updateEtat :: "+ e.toString());
		}
		finally{
        	try{
        		res.close();
				// statement.close();
			}
			catch (Exception e) {
				// ignore the exception
			}
            try{
				con.close();
			}catch (Exception e) {
				// ignore the exception
			}
		}
	}



	/** ************************************************************************************
	*	METHODES UTILITAIRES
	*
	**  ************************************************************************************/

	/**
	 * method to convert a GregrianCalendar date to a String
	 * @parame the GregrianCalendar date
	 * @return the date as a String
	 */
	public static String date2string(GregorianCalendar cal) {
		DateFormat df;
		df = DateFormat.getDateTimeInstance(DateFormat.MEDIUM,
                                            DateFormat.MEDIUM, m_locale);
		return df.format(cal.getTime());
	}


    	/**
	 * method to convert a GregrianCalendar date to a String
	 * @parame the GregrianCalendar date
	 * @return the date as a String
	 */
	public static String date2shortString(GregorianCalendar cal) {
		DateFormat df;
		df = DateFormat.getDateInstance(DateFormat.MEDIUM, m_locale);
		return df.format(cal.getTime());
	}


	/**
	 * method to convert a string date to a GregorianCalendar date.
	 * The method throws a ParseException if the string cannot be converted.
	 * @param the date as a String
	 * @return the date as a GregorianCalendar
	 */
	public static GregorianCalendar string2date(String sDate) throws ParseException {
		DateFormat df;
		GregorianCalendar ret = new GregorianCalendar();
		df = DateFormat.getDateTimeInstance(DateFormat.MEDIUM,
                                            DateFormat.MEDIUM, m_locale);
		java.util.Date date = df.parse(sDate);
		//System.err.println("string2date " + date.toString());
		ret.setTime(date);
		//System.err.println("string2date " + ret.get(Calendar.DATE)+"." + ret.get(Calendar.MONTH)+"." + ret.get(Calendar.YEAR) );
		return ret;
	}

    /**
	 * method to convert a string date to a GregorianCalendar date.
	 * The method throws a ParseException if the string cannot be converted.
	 * @param the date as a String
	 * @return the date as a GregorianCalendar
	 */
	public static GregorianCalendar shortString2date(String sDate) throws ParseException {
		DateFormat df;
		GregorianCalendar ret = new GregorianCalendar();
		df = DateFormat.getDateInstance(DateFormat.MEDIUM, m_locale);
		java.util.Date date = df.parse(sDate);
		//System.err.println("string2date " + date.toString());
		ret.setTime(date);
		//System.err.println("string2date " + ret.get(Calendar.DATE)+"." + ret.get(Calendar.MONTH)+"." + ret.get(Calendar.YEAR) );
		return ret;
	}

	/**
	 * method to convert a string date to a sqlDate date.
	 * The method throws a ParseException if the string cannot be converted.
	 * @param the date as a String
	 * @return the date as a sqlDate
	 */
	public static java.sql.Timestamp string2sqlTimestamp(String sDate) throws ParseException {
		DateFormat df;
		//df = DateFormat.getDateInstance(DateFormat.MEDIUM);
        df = DateFormat.getDateTimeInstance(DateFormat.MEDIUM, DateFormat.MEDIUM, m_locale);
		java.util.Date uDate = df.parse(sDate);
		//System.err.println("string2sqlDate " + uDate.toString());
		java.sql.Timestamp sqlTimestamp = new java.sql.Timestamp(uDate.getTime());  // Date to SQLDate
		return sqlTimestamp;
	}

    /**
	 * method to convert a string date to a sqlDate date.
	 * The method throws a ParseException if the string cannot be converted.
	 * @param the date as a String
	 * @return the date as a sqlDate
	 */
	public static java.sql.Timestamp shortString2sqlTimestamp(String sDate) throws ParseException {
		DateFormat df;
		//df = DateFormat.getDateInstance(DateFormat.MEDIUM);
        df = DateFormat.getDateInstance(DateFormat.MEDIUM, m_locale);
		java.util.Date uDate = df.parse(sDate);
		//System.err.println("string2sqlDate " + uDate.toString());
		java.sql.Timestamp sqlTimestamp = new java.sql.Timestamp(uDate.getTime());  // Date to SQLDate
		return sqlTimestamp;
	}

    public static java.sql.Timestamp date2Timestamp(GregorianCalendar gDate) {
        java.util.Date date= gDate.getTime();
        return new java.sql.Timestamp( date.getTime() );
    }

  /**
   * method to update the poolname
   * is invoked from listesChannelContentDefinition
   * @param String pool
   */
  public static void updatePoolName(String pool) {
    c_pool = pool;
  }
 
 


  /**
   * method to clear the cache
   * is invoked from write and delete
   * @param CmsObject cms
   */

  private void clearcache(CmsObject cms) {

    // first the element cache
    cms.getOnlineElementCache().clearCache();

    URL u = null;
    String p = ((HttpServletRequest)
                cms.getRequestContext().getRequest().getOriginalRequest()
                ).getServletPath();
    String s = ((HttpServletRequest)
                cms.getRequestContext().getRequest().getOriginalRequest()
                ).getServerName();
    try {
        u = new URL("http://" +s +p
                    +"/system/workplace/action/login.html?_clearcache=all");
        URLConnection uc = u.openConnection();
        uc.connect();
        InputStream in = uc.getInputStream();
        //System.err.println("clearcache o.k.!");
    } catch(MalformedURLException e) {
        System.err.println("Exception1: " +e.getMessage() );
    } catch (IOException e) {
        System.err.println("Exception2: " +e.getMessage() );
    }

  }



}

